#=========
# Packages
#=========

library(DESeq2)
library(tidyverse)
library(msigdbr)
library(cowplot)
library(ggrepel)
library(ggplot2)
library(viridis)


#========
# Functions
#========

getCodingGenes <- function() {
  require(biomaRt)
  mart <- useMart(biomart = "ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl", host = 'https://www.ensembl.org')
  genes <- biomaRt::getBM(attributes = c("ensembl_gene_id", "external_gene_name", "chromosome_name","transcript_biotype"), 
                          filters = c("transcript_biotype","chromosome_name"),
                          values = list("protein_coding",c(1:22)),
                          mart = mart)
  return(genes)
}


msigdb_enrichment <- function(pathway_DB, pathways, background_genes, genes_of_interest){
  # Taken from: https://github.com/blekhmanlab/host_gene_microbiome_interactions
  # Author: Sambhawa Priya
  # set min and max genes to allow pathways for testing
  min_genes <- 25
  max_genes <- 85
  overlap_genes <- 5
  
  enrichment_list <- list()
  
  for(i in 1:length(pathways)){
    
    pathway <- pathways[i]
    
    ## genes in this pathway
    pathway_gene_set <- pathway_DB[pathway_DB$gs_name == pathway,]$human_ensembl_gene
    length(pathway_gene_set) 
    ## If the criteria for min and max #genes in a given pathway is not satified, 
    ## skip testing the current pathway
    if(length(pathway_gene_set) < min_genes || length(pathway_gene_set) > max_genes) next
    
    ## The contingency table
    ## Inspired by: https://www.pathwaycommons.org/guide/primers/statistics/fishers_exact_test/
    ##                    genes_of_interest genes_NOT_of_interest  Total
    ## In_pathway               x                 m - x              m
    ## Not_in_pathway         k - x             n - (k - x)          n
    ## Total                    k                 m + n - k          m + n
    
    ## m, #overlapping genes in this pathway and background genes for IBS
    m <- length(intersect(background_genes,pathway_gene_set))
    m ## 11
    
    ## n, #genes in background but not in pathway 
    n <- length(setdiff(background_genes,pathway_gene_set))
    n #12502
    
    ## x, #genes of interest in pathway 
    x <- length(intersect(pathway_gene_set,genes_of_interest))
    x ## 1
    ## If the overlap between genes of interest and the pathway is less than the overlap cut-off, 
    ## then skip testing the current pathway
    if(x < overlap_genes) next 
    
    # if(x > 0){
    #   gene_names_in_pathway = noquote(paste(intersect(pathway_gene_set,genes_of_interest), collapse = ","))
    # } else {
    #   gene_names_in_pathway = ""
    # }
    ## Extract list of genes in the genes of interest that are included in the pathway. 
    gene_names_in_pathway = noquote(paste(intersect(pathway_gene_set,genes_of_interest), collapse = ","))
    
    ## k, total #genes in genes list of interest
    k <- length(genes_of_interest)
    k #839
    
    ## Build the contigency table
    contingency_table <- matrix(c(x, k-x, m-x, n-(k-x)), #matrix is filled along the column by default
                                nrow = 2, ncol = 2, 
                                dimnames = list(c("In_pathway","Not_in_pathway"),
                                                c("Genes_of_interest","Genes_NOT_of_interest"))
    )
    
    contingency_table
    
    
    fisher_result <- fisher.test(contingency_table, alternative = "greater")
    
    ## save details in a dataframe
    enrichment_result_df <- data.frame( pathway = pathway,
                                        BG_genes = length(background_genes),
                                        genes_in_pathway = length(pathway_gene_set),
                                        genes_in_path_and_BG = m,
                                        genes_of_interest = k,
                                        genes_of_interest_in_pathway = x,
                                        gene_names_in_pathway = gene_names_in_pathway,
                                        ## fill in contingency table entries: x, k-x, m-x, n-(k-x) for z-score computation.
                                        cont_n1 = x,
                                        cont_n2 = k-x,
                                        cont_n3 = m-x,
                                        cont_n4 = n-(k-x),
                                        CI_95 = paste0(signif(fisher_result$conf.int[1],5),"-",signif(fisher_result$conf.int[2],5)),
                                        odds_ratio = unname(fisher_result$estimate),
                                        p_val = fisher_result$p.value
    )
    
    enrichment_list[[i]] <- enrichment_result_df
    
  }
  
  return(enrichment_list)
  
}


plot_enrichment_results <- function(res_up, res_down, names){
  data <- lapply(list(res_up,res_down), function(x){
    x$gene_ratio <- x$genes_of_interest_in_pathway/x$genes_in_pathway
    return(x)
  })
  names(data) <- names
  plt_data <- bind_rows(data, .id = "set") 
  plt_data$set <- factor(plt_data$set, levels = c(names[1], names[2]))
  
  ggplot(plt_data, aes(x = genes_of_interest_in_pathway, y = fct_reorder2(pathway, set, p_adj), fill = set))+
    geom_bar(stat = "identity")+
    facet_grid(rows = vars(set), scales = "free_y", space = "free")+
    scale_fill_manual(labels = c(paste0("", names[1]), 
                                 paste0("", names[2])),
                      values  = c("#b12b2e", "#6697ea"))+
    labs(y = "Pathway", x = "Genes of Interest In Pathway")+
    theme_light()
}

plotPCA <- function(object, intgroup="condition", ntop=500, returnData=FALSE)
{ #STOLEN FROM DESEQ2 source code 
  
  # calculate the variance for each gene
  rv <- rowVars(assay(object))
  
  # select the ntop genes by variance
  select <- order(rv, decreasing=TRUE)[seq_len(min(ntop, length(rv)))]
  
  # perform a PCA on the data in assay(x) for the selected genes
  pca <- prcomp(t(assay(object)[select,]))
  
  # the contribution to the total variance for each component
  percentVar <- pca$sdev^2 / sum( pca$sdev^2 )
  
  if (!all(intgroup %in% names(colData(object)))) {
    stop("the argument 'intgroup' should specify columns of colData(dds)")
  }
  
  intgroup.df <- as.data.frame(colData(object)[, intgroup, drop=FALSE])
  
  # add the intgroup factors together to create a new grouping factor
  group <- if (length(intgroup) > 1) {
    factor(apply( intgroup.df, 1, paste, collapse=":"))
  } else {
    colData(object)[[intgroup]]
  }
  
  # assembly the data for the plot
  d <- data.frame(PC1=pca$x[,1], PC2=pca$x[,2], group=group, intgroup.df, name=colnames(object))
  
  if (returnData) {
    attr(d, "percentVar") <- percentVar[1:2]
    return(d)
  }
  
  ggplot(data=d, aes_string(x="PC1", y="PC2", color="group")) + geom_point(size=3, alpha = 0.5) + 
    xlab(paste0("PC1: ",round(percentVar[1] * 100),"% variance")) +
    ylab(paste0("PC2: ",round(percentVar[2] * 100),"% variance")) +
    coord_fixed()
}

EnsemblIDtoSymbol<-function(ensembl_id){
    return(coding_genes$external_gene_name[coding_genes$ensembl_gene_id==ensembl_id])
  }


#========================================
# Human Gene Expression 
#========================================

#Read in the count data 

raw_counts <- read.delim("./input_data/count_matrix.tsv")
metadata <- read.delim("./input_data/metadata_Rformatted.tsv")

bacterial_load <- read.delim("./input_data/kingdom_counts.csv")

#=======================================
# Preliminary Processing data and metadata
#=======================================

# Add High Bacterial Variable 


high_bac_samples <- c("La98",
                      "La70",
                      "La96",
                      "La106",
                      "La110",
                      "Lasse20")

metadata$bac_load <- ifelse(metadata$RNA_seq_ID %in% high_bac_samples, "high_bacteria", "low_bacteria")


# Extract Patient_ID 
metadata <- 
  metadata %>% 
  separate(pt_ID, sep = "-", into = c("Patient_ID")) #Get patient ID column 

#Add Colsums to Metadata 

metadata <- left_join(metadata,
                      enframe(colSums(raw_counts[2:ncol(raw_counts)]/1000000), 
                              name = "RNA_seq_ID", value = "raw_counts_M"),
                      by = "RNA_seq_ID")

metadata_subset <- subset(metadata, Type != "Control" & (Patient_ID!="030"|Patient_ID!="055")) 
#Patients 30 and 55 were not in the study  

#Create a metadata column for CRC and Healthy

metadata$cancer <- gsub("Healthy|Paired", "No_CRC", metadata$Type)
metadata$cancer <- factor(metadata$cancer, levels = c("CRC", "No_CRC"))

# Make basic metadata and counts for DESeq2

cts <- column_to_rownames(raw_counts, "gene")
coldata <- column_to_rownames(metadata, "RNA_seq_ID")

#Filter out controls 
coldata <- filter(coldata, row.names(coldata) != "La125")
cts <- cts[,colnames(cts) != "La125"]

#Filter metadata for samples not sequenced 

coldata <- filter(coldata, row.names(coldata) %in% colnames(cts))

#Order cts and coldata
cts <- cts[ , order(names(cts))]
coldata <- coldata[order(row.names(coldata)), ,drop=F]

# Filter only CRC samples

cts <- cts[,row.names(coldata)[coldata$cancer == "CRC"]]
coldata <- subset(coldata, cancer == "CRC")

#=========================================
# Filter only Protein-coding genes 
#=========================================

# Only include protein-coding genes in count data  

coding_genes <- getCodingGenes()

#Filter only coding genes in counts 

cts <- cts[row.names(cts) %in% coding_genes$ensembl_gene_id,]


#=====================================================================================
# Q1. What genes/pathways are significantly expressed in High Bacterial Load vs Low Bacterial Load 
#=====================================================================================


# Format the data to DESeq2
dds <- DESeqDataSetFromMatrix(countData=cts,
                              colData=coldata,
                              design= ~ bac_load)

# Normalization and preprocessing
dds <- DESeq(dds)

#Define the contrast 
contrast <- c("bac_load", c("high_bacteria", "low_bacteria"))

# Generate the results
res <- results(dds, contrast=contrast)
# shrink fold changes for lowly expressed genes
# use ashr so we can use `contrast` as conversion to coef is not trivial
# see https://bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.html #extended-section-on-shrinkage-estimators
#res <- lfcShrink(dds, contrast=contrast, res=res, type="ashr")

# sort by p-value
res <- res[order(res$padj),]
summary(res, alpha = 0.05)

# Get significant genes
res_sig <- data.frame(subset(res, padj < 0.05 & abs(log2FoldChange) >=2))



#Get top X up and downregulated genes for labelling 

top_genes_BacLoad <- c(row.names(subset(res_sig[order(res_sig$padj),], log2FoldChange>2))[1:20],
                    row.names(subset(res_sig[order(res_sig$padj),], log2FoldChange<(-2)))[1:20])



# volcano plot 

df_volcano <- 
    data.frame(res) %>%
      rownames_to_column("ensembl_gene_id") %>%
      left_join(coding_genes, by = "ensembl_gene_id") %>%
      mutate("Expression" = case_when(log2FoldChange >= 2 & padj<= 0.05 ~ "High Bacterial Load",
                                      log2FoldChange <= (-2) & padj <= 0.05 ~ "Low Bacterial Load",
                                      TRUE ~ "Unchanged"))



plt_volcano <- ggplot(df_volcano, aes(x = log2FoldChange, y = -log10(pvalue), color = Expression))+
  geom_point(alpha = 0.5)+
  xlim(-9,9)+
  geom_vline(xintercept = c(-2,2), linetype = "dashed")+
  geom_hline(yintercept = -log10(0.05), linetype = "dashed")+
  scale_color_manual(values = c("High Bacterial Load" = "#b12b2e",
                                "Low Bacterial Load" = "#6697ea",
                                "Not significant" = "#bebebe"))+
  labs(title = "High vs. Low Bacterial Load")+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_BacLoad,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)+
  
  theme_light()

plt_volcano




#MA plot 

df_MA <- 
  plotMA(res, returnData=T) %>%
    mutate("significant" = case_when(lfc >= 2 & isDE ~ "High Bacterial Activity",
                                     lfc <= -2 & isDE  ~ "Low Bacterial Activity",
                                     isDE & lfc<2 & lfc>-2 ~ "Significant, but low LFC",
                                     TRUE ~ "Not significant")) %>%
    mutate("ensembl_gene_id"= row.names(res)) %>%
    left_join(coding_genes[, c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") 

plt_MA<-
  ggplot(df_MA, aes(x = log2(mean), y = lfc, color = significant))+
  geom_point( alpha = 0.5)+
  scale_color_manual(values = c("High Bacterial Activity" = "#b12b2e",
                                "Low Bacterial Activity" = "#6697ea",
                                "Significant, but low LFC" = "#7f7f7f",
                                "Not significant" = "#bebebe"))+
  ylim(-10,10)+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_BacLoad,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)+
  geom_hline(yintercept = 2, linetype = "dashed")+
  geom_hline(yintercept = -2, linetype = "dashed")+
  labs(x="Log2 Mean Expression", y = "Log2 Fold Change", title ="Differentially Expressed Genes: Bacterial Activity",
       color = "Differential Expression")+
  theme_light()+
  guides(color=guide_legend(nrow=2,byrow=TRUE,
                            override.aes = list(size=6)))+
    theme(text = element_text(size = 12),
          legend.position = "bottom",
          legend.text = element_text(size = 10),
          legend.title = element_blank())



#plt_MA

ggsave("./generated_figures/plt_volcano_BacLoad.png", plt_volcano, width = 7, height = 5)
ggsave("./generated_figures/plt_MA_BacLoad.png", plt_MA, width = 6.5, height = 4.5)



#==================================
# Which Pathways are enriched?
#==================================

##### Get Genes of Interest  ######

background_genes <- row.names(cts)

interesting_genes_UPBacLoad <- row.names(res_sig)[res_sig$log2FoldChange>0]
interesting_genes_DOWNBacLoad <- row.names(res_sig)[res_sig$log2FoldChange<0] 

##### Get MSigDB pathways    ##################

# Stolen from: https://github.com/blekhmanlab/host_gene_microbiome_interactions/blob/main/sparseCCA/msigdb_enrichment_sparseCCA.R

## Let's focus first on C2 subcollection canonical pathways 
## http://software.broadinstitute.org/gsea/msigdb/collection_details.jsp#C2
## This includes gene sets from BioCarta, KEGG, PID, and Reactome. 

msigdb_C2 <- msigdbr(species = "Homo sapiens", category = "C2")
class(msigdb_C2) #[1] "tbl_df"     "tbl"        "data.frame"
msigdb_C2 <- as.data.frame(msigdb_C2)
table(msigdb_C2$gs_subcat)
# CGP          CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 363007        3697        4775       12783        8058       85647
## Only keep canonical pathways, or in other words, remove CGP (Chemical and genetic perturbations)

msigdb_C2_CP <- msigdb_C2[msigdb_C2$gs_subcat!= "CGP",]
table(msigdb_C2_CP$gs_subcat)
# CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 3697        4775       12783        8058       85647 
dim(msigdb_C2_CP)
# [1] 114960      9
length(unique(msigdb_C2_CP$gs_name))
# 2199 pathways

## Only keep pathway DBs that we want to test gene sets against 
table(msigdb_C2_CP$gs_subcat)
# CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 3697        4775       12783        8058       85647 

## Pathway DB of interest: KEGG and PID
path_DB <- c("CP:KEGG","CP:PID", "CP:REACTOME")
msigdb_C2_CP <- msigdb_C2_CP[(msigdb_C2_CP$gs_subcat %in% path_DB), ]
table(msigdb_C2_CP$gs_subcat)
# CP:KEGG      CP:PID 
# 12783        8058  


## unique pathways in DBs of interest
msigdb_C2_CP_unique_pathways <- unique(msigdb_C2_CP$gs_name)

length(msigdb_C2_CP_unique_pathways) 



##### Run the Enrichment ####################

#Enriched in High BacLoad 

HighBac_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                  background_genes, interesting_genes_UPBacLoad)

## convert the list to dataframe
HighBac_pathways <- do.call(rbind.data.frame, HighBac_pathways)
dim(HighBac_pathways) 

##sort by pval
HighBac_pathways <- HighBac_pathways[order(HighBac_pathways$p_val),]
## MHT correction (FDR)
HighBac_pathways$p_adj <- p.adjust(HighBac_pathways$p_val, method = "BH")

length(which(HighBac_pathways$p_adj < 0.05))

HighBac_pathways <- HighBac_pathways[HighBac_pathways$p_adj < 0.05,]
dim(HighBac_pathways) 

# Enriched in Healthy 

LowBac_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                      background_genes, interesting_genes_DOWNBacLoad)

## convert the list to dataframe
LowBac_pathways <- do.call(rbind.data.frame, LowBac_pathways)
dim(LowBac_pathways) 

##sort by pval
LowBac_pathways <- LowBac_pathways[order(LowBac_pathways$p_val),]

## MHT correction (FDR)
LowBac_pathways$p_adj <- p.adjust(LowBac_pathways$p_val, method = "BH")

length(which(LowBac_pathways$p_adj < 0.05))

LowBac_pathways <- LowBac_pathways[LowBac_pathways$p_adj < 0.05,]
dim(LowBac_pathways) 


# Make a quick plot 

plt_enrichment_BacLoad <- plot_enrichment_results(HighBac_pathways, LowBac_pathways, 
                                              names = c("High Bacterial Activity", "Low Bacterial Activity"))+
  theme(axis.text.y = element_text(size = 10),
        legend.position = "top",
        legend.title = element_blank())

plt_enrichment_BacLoad


ggsave("./generated_figures/plt_enrichment_BacLoad.png", plt_enrichment_BacLoad, height = 5, width = 8)


#=====================================
# Add gene symbols to pathway output 
#=====================================

HighBac_pathways$gene_symbols_in_pathway <- lapply(HighBac_pathways$gene_names_in_pathway, function(x){
  genes <- unlist(str_split(x, ","))
  out <- lapply(genes, EnsemblIDtoSymbol)
  return(paste(out, collapse = ","))
  })

LowBac_pathways$gene_symbols_in_pathway <- lapply(LowBac_pathways$gene_names_in_pathway, function(x){
  genes <- unlist(str_split(x, ","))
  out <- lapply(genes, EnsemblIDtoSymbol)
  return(paste(out, collapse = ","))
})


# Export the Results 

out_DEgenes <- 
  res_sig %>%
    rownames_to_column("ensembl_gene_id") %>%
    left_join(coding_genes[,c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") %>%
    relocate("external_gene_name", .after = "ensembl_gene_id") %>%
    arrange(log2FoldChange)

write_csv(out_DEgenes, "./generated_data/DESeq2_BacLoadCRC_sigGenes.csv")

write_csv(HighBac_pathways, "./generated_data/Enrichment_HighBac_pathways.csv")
write_csv(LowBac_pathways, "./generated_data/Enrichment_LowBac_pathways.csv")


