#=========
# Packages
#=========

library(DESeq2)
library(tidyverse)
library(msigdbr)
library(cowplot)
library(ggrepel)
library(ggplot2)
library(viridis)


#========
# Functions
#========

getCodingGenes <- function() {
  require(biomaRt)
  mart <- useMart(biomart = "ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl", host = 'https://www.ensembl.org')
  genes <- biomaRt::getBM(attributes = c("ensembl_gene_id", "external_gene_name", "chromosome_name","transcript_biotype"), 
                          filters = c("transcript_biotype","chromosome_name"),
                          values = list("protein_coding",c(1:22)),
                          mart = mart)
  return(genes)
}


msigdb_enrichment <- function(pathway_DB, pathways, background_genes, genes_of_interest){
  # Taken from: https://github.com/blekhmanlab/host_gene_microbiome_interactions
  # Author: Sambhawa Priya
  # set min and max genes to allow pathways for testing
  min_genes <- 25
  max_genes <- 85
  overlap_genes <- 5
  
  enrichment_list <- list()
  
  for(i in 1:length(pathways)){
    
    pathway <- pathways[i]
    
    ## genes in this pathway
    pathway_gene_set <- pathway_DB[pathway_DB$gs_name == pathway,]$human_ensembl_gene
    length(pathway_gene_set) 
    ## If the criteria for min and max #genes in a given pathway is not satified, 
    ## skip testing the current pathway
    if(length(pathway_gene_set) < min_genes || length(pathway_gene_set) > max_genes) next
    
    ## The contingency table
    ## Inspired by: https://www.pathwaycommons.org/guide/primers/statistics/fishers_exact_test/
    ##                    genes_of_interest genes_NOT_of_interest  Total
    ## In_pathway               x                 m - x              m
    ## Not_in_pathway         k - x             n - (k - x)          n
    ## Total                    k                 m + n - k          m + n
    
    ## m, #overlapping genes in this pathway and background genes for IBS
    m <- length(intersect(background_genes,pathway_gene_set))
    m ## 11
    
    ## n, #genes in background but not in pathway 
    n <- length(setdiff(background_genes,pathway_gene_set))
    n #12502
    
    ## x, #genes of interest in pathway 
    x <- length(intersect(pathway_gene_set,genes_of_interest))
    x ## 1
    ## If the overlap between genes of interest and the pathway is less than the overlap cut-off, 
    ## then skip testing the current pathway
    if(x < overlap_genes) next 
    
    # if(x > 0){
    #   gene_names_in_pathway = noquote(paste(intersect(pathway_gene_set,genes_of_interest), collapse = ","))
    # } else {
    #   gene_names_in_pathway = ""
    # }
    ## Extract list of genes in the genes of interest that are included in the pathway. 
    gene_names_in_pathway = noquote(paste(intersect(pathway_gene_set,genes_of_interest), collapse = ","))
    
    ## k, total #genes in genes list of interest
    k <- length(genes_of_interest)
    k #839
    
    ## Build the contigency table
    contingency_table <- matrix(c(x, k-x, m-x, n-(k-x)), #matrix is filled along the column by default
                                nrow = 2, ncol = 2, 
                                dimnames = list(c("In_pathway","Not_in_pathway"),
                                                c("Genes_of_interest","Genes_NOT_of_interest"))
    )
    
    contingency_table
    
    
    fisher_result <- fisher.test( contingency_table, alternative = "greater")
    
    ## save details in a dataframe
    enrichment_result_df <- data.frame( pathway = pathway,
                                        BG_genes = length(background_genes),
                                        genes_in_pathway = length(pathway_gene_set),
                                        genes_in_path_and_BG = m,
                                        genes_of_interest = k,
                                        genes_of_interest_in_pathway = x,
                                        gene_names_in_pathway = gene_names_in_pathway,
                                        ## fill in contingency table entries: x, k-x, m-x, n-(k-x) for z-score computation.
                                        cont_n1 = x,
                                        cont_n2 = k-x,
                                        cont_n3 = m-x,
                                        cont_n4 = n-(k-x),
                                        CI_95 = paste0(signif(fisher_result$conf.int[1],5),"-",signif(fisher_result$conf.int[2],5)),
                                        odds_ratio = unname(fisher_result$estimate),
                                        p_val = fisher_result$p.value
    )
    
    enrichment_list[[i]] <- enrichment_result_df
    
  }
  
  return(enrichment_list)
  
}


plot_enrichment_results <- function(res_up, res_down, names){
  data <- lapply(list(res_up,res_down), function(x){
    x$gene_ratio <- x$genes_of_interest_in_pathway/x$genes_in_pathway
    return(x)
  })
  names(data) <- names
  plt_data <- bind_rows(data, .id = "set") 
  plt_data$set <- factor(plt_data$set, levels = c(names[1], names[2]))
  
  ggplot(plt_data, aes(x = genes_of_interest_in_pathway, y = fct_reorder2(pathway, set, p_adj), fill = set))+
    geom_bar(stat = "identity")+
    facet_grid(rows = vars(set), scales = "free_y", space = "free")+
    scale_fill_manual(labels = c(paste0("Enriched in ", names[1]), 
                                 paste0("Enriched in ", names[2])),
                      values  = c("#b12b2e", "#6697ea"))+
    labs(y = "Pathway", x = "Genes of Interest In Pathway")+
    theme_light()
}

plotPCA <- function(object, intgroup="condition", ntop=500, returnData=FALSE)
{ #STOLEN FROM DESEQ2 source code 
  
  # calculate the variance for each gene
  rv <- rowVars(assay(object))
  
  # select the ntop genes by variance
  select <- order(rv, decreasing=TRUE)[seq_len(min(ntop, length(rv)))]
  
  # perform a PCA on the data in assay(x) for the selected genes
  pca <- prcomp(t(assay(object)[select,]))
  
  # the contribution to the total variance for each component
  percentVar <- pca$sdev^2 / sum( pca$sdev^2 )
  
  if (!all(intgroup %in% names(colData(object)))) {
    stop("the argument 'intgroup' should specify columns of colData(dds)")
  }
  
  intgroup.df <- as.data.frame(colData(object)[, intgroup, drop=FALSE])
  
  # add the intgroup factors together to create a new grouping factor
  group <- if (length(intgroup) > 1) {
    factor(apply( intgroup.df, 1, paste, collapse=":"))
  } else {
    colData(object)[[intgroup]]
  }
  
  # assembly the data for the plot
  d <- data.frame(PC1=pca$x[,1], PC2=pca$x[,2], group=group, intgroup.df, name=colnames(object))
  
  if (returnData) {
    attr(d, "percentVar") <- percentVar[1:2]
    return(d)
  }
  
  ggplot(data=d, aes_string(x="PC1", y="PC2", color="group")) + geom_point(size=3, alpha = 0.5) + 
    xlab(paste0("PC1: ",round(percentVar[1] * 100),"% variance")) +
    ylab(paste0("PC2: ",round(percentVar[2] * 100),"% variance")) +
    coord_fixed()
}

#========================================
# Human Gene Expression 
#========================================

#Read in the count data 

raw_counts <- read.delim("./input_data/count_matrix.tsv")
metadata <- read.delim("./input_data/metadata_Rformatted.tsv")


#=======================================
# Preliminary Processing data and metadata
#=======================================

# Extract Patient_ID 
metadata <- 
  metadata %>% 
  separate(pt_ID, sep = "-", into = c("Patient_ID")) #Get patient ID column 

#Add Colsums to Metadata 

metadata <- left_join(metadata,
                      enframe(colSums(raw_counts[2:ncol(raw_counts)]/1000000), 
                              name = "RNA_seq_ID", value = "raw_counts_M"),
                      by = "RNA_seq_ID")

metadata_subset <- subset(metadata, Type != "Control" & (Patient_ID!="030"|Patient_ID!="055")) 
#Patients 30 and 55 were not in the study  

#Create a metadata column for CRC and Healthy

metadata$cancer <- gsub("Healthy|Paired", "No_CRC", metadata$Type)
metadata$cancer <- factor(metadata$cancer, levels = c("CRC", "No_CRC"))

# Make basic metadata and counts for DESeq2

cts <- column_to_rownames(raw_counts, "gene")
coldata <- column_to_rownames(metadata, "RNA_seq_ID")

#Filter out controls 
coldata <- filter(coldata, row.names(coldata) != "La125")
cts <- cts[,colnames(cts) != "La125"]

#Filter metadata for samples not sequenced 

coldata <- filter(coldata, row.names(coldata) %in% colnames(cts))

#Order cts and coldata
cts <- cts[ , order(names(cts))]
coldata <- coldata[order(row.names(coldata)), ,drop=F]

#=========================================
# Filter only Protein-coding genes 
#=========================================

# Only include protein-coding genes in count data  

coding_genes <- getCodingGenes()

#Filter only coding genes in counts 

cts <- cts[row.names(cts) %in% coding_genes$ensembl_gene_id,]


#=====================================================================================
# Q1. What genes/pathways are significantly expressed in CRC vs Paired-Normal/Healthy
#=====================================================================================


# Format the data to DESeq2
dds <- DESeqDataSetFromMatrix(countData=cts,
                              colData=coldata,
                              design= ~ cancer)

# Normalization and preprocessing
dds <- DESeq(dds)

#Define the contrast 
contrast <- c("cancer", c("CRC", "No_CRC"))

# Generate the results
res <- results(dds, contrast=contrast)
# shrink fold changes for lowly expressed genes
# use ashr so we can use `contrast` as conversion to coef is not trivial
# see https://bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.html #extended-section-on-shrinkage-estimators
#res <- lfcShrink(dds, contrast=contrast, res=res, type="ashr")

# sort by p-value
res <- res[order(res$padj),]
summary(res, alpha = 0.05)

# Get significant genes
res_sig <- data.frame(subset(res, padj < 0.05 & abs(log2FoldChange) >=2))



# volcano plot 

df_volcano <- tibble(data.frame(res)) %>%
                mutate("Expression" = case_when(log2FoldChange >= 2 & padj<= 0.05 ~ "Up-regulated in CRC",
                                                 log2FoldChange <= (-2) & padj <= 0.05 ~ "Down-regulated in CRC",
                                                 TRUE ~ "Unchanged"))



plt_volcano <- ggplot(df_volcano, aes(x = log2FoldChange, y = -log10(pvalue), color = Expression))+
  geom_point(alpha = 0.5)+
  xlim(-9,9)+
  geom_vline(xintercept = c(-2,2), linetype = "dashed")+
  geom_hline(yintercept = -log10(0.05), linetype = "dashed")+
  scale_color_manual(values = c("Up-regulated in CRC" = "#b12b2e",
                                "Down-regulated in CRC" = "#6697ea",
                                "Not significant" = "#bebebe"))+
  labs(title = "CRC vs Non-CRC")+
  
  theme_light()

plt_volcano

#MA plot 

df_MA <- plotMA(res, returnData=T) %>%
          mutate("significant" = case_when(lfc >= 2 & isDE ~ "Up-regulated in CRC",
                                           lfc <= -2 & isDE  ~ "Down-regulated in CRC",
                                           isDE & lfc<2 & lfc>-2 ~ "Significant, but low LFC",
                                           TRUE ~ "Unchanged"))

plt_MA<-
ggplot(df_MA, aes(x = log2(mean), y = lfc, color = significant))+
  geom_point( alpha = 0.5)+
  scale_color_manual(values = c("Up-regulated in CRC" = "#b12b2e",
                                "Down-regulated in CRC" = "#6697ea",
                                "Significant, but low LFC" = "#7f7f7f",
                                "Not significant" = "#bebebe"))+
  ylim(-10,10)+
  geom_hline(yintercept = 2, linetype = "dashed")+
  geom_hline(yintercept = -2, linetype = "dashed")+
  labs(x="Log2 Mean Expression", y = "Log2 Fold Change", title ="CRC vs Non-CRC",
       color = "Differential Expression")+
  theme_light()


ggsave("./generated_figures/plt_volcano_CRC.png", plt_volcano, width = 5, height = 5)
ggsave("./generated_figures/plt_MA_CRC.png", plt_MA, width = 7, height = 4)

#==================================
# Which Pathways are enriched?
#==================================

##### Get Genes of Interest  ######

background_genes <- row.names(cts)

interesting_genes_UPcrc <- row.names(res_sig)[res_sig$log2FoldChange>0]
interesting_genes_DOWNcrc <- row.names(res_sig)[res_sig$log2FoldChange<0] 

##### Get MSigDB pathways    ##################

# Stolen from: https://github.com/blekhmanlab/host_gene_microbiome_interactions/blob/main/sparseCCA/msigdb_enrichment_sparseCCA.R

## Let's focus first on C2 subcollection canonical pathways 
## http://software.broadinstitute.org/gsea/msigdb/collection_details.jsp#C2
## This includes gene sets from BioCarta, KEGG, PID, and Reactome. 

msigdb_C2 <- msigdbr(species = "Homo sapiens", category = "C2")
class(msigdb_C2) #[1] "tbl_df"     "tbl"        "data.frame"
msigdb_C2 <- as.data.frame(msigdb_C2)
table(msigdb_C2$gs_subcat)
# CGP          CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 363007        3697        4775       12783        8058       85647
## Only keep canonical pathways, or in other words, remove CGP (Chemical and genetic perturbations)

msigdb_C2_CP <- msigdb_C2[msigdb_C2$gs_subcat!= "CGP",]
table(msigdb_C2_CP$gs_subcat)
# CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 3697        4775       12783        8058       85647 
dim(msigdb_C2_CP)
# [1] 114960      9
length(unique(msigdb_C2_CP$gs_name))
# 2199 pathways

## Only keep pathway DBs that we want to test gene sets against 
table(msigdb_C2_CP$gs_subcat)
# CP CP:BIOCARTA     CP:KEGG      CP:PID CP:REACTOME 
# 3697        4775       12783        8058       85647 

## Pathway DB of interest: KEGG and PID
path_DB <- c("CP:KEGG","CP:PID", "CP:REACTOME")
msigdb_C2_CP <- msigdb_C2_CP[(msigdb_C2_CP$gs_subcat %in% path_DB), ]
table(msigdb_C2_CP$gs_subcat)
# CP:KEGG      CP:PID 
# 12783        8058  


## unique pathways in DBs of interest
msigdb_C2_CP_unique_pathways <- unique(msigdb_C2_CP$gs_name)

length(msigdb_C2_CP_unique_pathways) 



##### Run the Enrichment ####################

#Enriched in CRC 

CRC_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                          background_genes, interesting_genes_UPcrc)

## convert the list to dataframe
CRC_pathways <- do.call(rbind.data.frame, CRC_pathways)
dim(CRC_pathways) 

##sort by pval
CRC_pathways <- CRC_pathways[order(CRC_pathways$p_val),]
## MHT correction (FDR)
CRC_pathways$p_adj <- p.adjust(CRC_pathways$p_val, method = "BH")

length(which(CRC_pathways$p_adj < 0.05))

CRC_pathways <- CRC_pathways[CRC_pathways$p_adj < 0.05,]
dim(CRC_pathways) 

# Enriched in Healthy 

Healthy_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                  background_genes, interesting_genes_DOWNcrc)

## convert the list to dataframe
Healthy_pathways <- do.call(rbind.data.frame, Healthy_pathways)
dim(Healthy_pathways) 

##sort by pval
Healthy_pathways <- Healthy_pathways[order(Healthy_pathways$p_val),]

## MHT correction (FDR)
Healthy_pathways$p_adj <- p.adjust(Healthy_pathways$p_val, method = "BH")

length(which(Healthy_pathways$p_adj < 0.05))

Healthy_pathways <- Healthy_pathways[Healthy_pathways$p_adj < 0.05,]
dim(Healthy_pathways) 


# Make a quick plot 

plt_enrichment_CRC <- plot_enrichment_results(CRC_pathways, Healthy_pathways, 
                                               names = c("CRC", "Healthy/Paired"))+
  theme(axis.text.y = element_text(size = 5))

plt_enrichment_CRC


ggsave("./generated_figures/plt_enrichment_CRC.png", plt_enrichment_CRC, height = 5, width = 8)

#=====================================================================================================
# Q2. Find significant DE genes in CRC (+/- *F. nucleatum* samples) to the effect of F. nucleatum on 
#     gene expression
#=====================================================================================================

#These CRC samples were identified (by HeaDs' analysis) as having elevated levels of F. nucleatum 

high_Fnucl <- c("La5",
                "La20",
                "La67",
                "La76",
                "La80",
                "La96",
                "La98",
                "La110",
                "La122")

#Make new coldata and metadata for the subset 

cts_CRC <- cts[colnames(cts) %in% row.names(coldata)[coldata$Type =="CRC"]]
coldata_CRC <- coldata[coldata$Type =="CRC", ]

# Add variable for high F. nucleatum 

coldata_CRC$high_Fnucl <- ifelse(row.names(coldata_CRC) %in% high_Fnucl, "high_Fnucl", "low_Fnucl")


# Make and run DESeq

dds_Fnucl <-  DESeqDataSetFromMatrix(countData=cts_CRC,
                                     colData=coldata_CRC,
                                     design= ~ high_Fnucl)

dds_Fnucl <- DESeq(dds_Fnucl)

#Define the contrast 
contrast <- c("high_Fnucl", c("high_Fnucl", "low_Fnucl"))

# Generate the res_Fnuclults
res_Fnucl <- results(dds_Fnucl, contrast=contrast)
# shrink fold changes for lowly expressed genes
# use ashr so we can use `contrast` as conversion to coef is not trivial
# see https://bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.html #extended-section-on-shrinkage-estimators
#res_Fnucl <- lfcShrink(dds_Fnucl, contrast=contrast, res=res_Fnucl, type="ashr")

# MA plot 
plotMA(res_Fnucl, ylim=c(-2,2))

# sort by p-value
res_Fnucl <- res_Fnucl[order(res_Fnucl$padj),]
summary(res_Fnucl, alpha = 0.05)

# Get significant genes
res_Fnucl_sig <- subset(res_Fnucl, padj < 0.05 & abs(log2FoldChange) >=2)


#Get top X up and downregulated genes for labelling 

top_genes_fuso <- c(row.names(subset(res_Fnucl_sig[order(res_Fnucl_sig$padj),], log2FoldChange>2))[1:20],
                    row.names(subset(res_Fnucl_sig[order(res_Fnucl_sig$padj),], log2FoldChange<(-2)))[1:20])

#MA plot 

df_MA_Fuso <- plotMA(res_Fnucl, returnData=T) %>%
  mutate("significant" = case_when(lfc >= 2 & isDE ~ "Up-regulated",
                                   lfc <= -2 & isDE  ~ "Down-regulated",
                                   isDE & lfc<2 & lfc>-2 ~ "Significant, but low LFC",
                                   TRUE ~ "Not significant")) %>%
  mutate("ensembl_gene_id"= row.names(res_Fnucl)) %>%
  left_join(coding_genes[, c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") 

plt_MA_Fuso<-
  ggplot(df_MA_Fuso, aes(x = log2(mean), y = lfc, color = significant))+
  geom_point( alpha = 0.5)+
  scale_color_manual(values = c("Up-regulated" = "#dfb5cc",
                                "Down-regulated" = "#b5a8f1",
                                "Significant, but low LFC" = "#7f7f7f",
                                "Not significant" = "#bebebe"))+
  ylim(-8,8)+
  geom_hline(yintercept = 2, linetype = "dashed")+
  geom_hline(yintercept = -2, linetype = "dashed")+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_fuso,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)+
  labs(x="Log2 Mean Expression", y = "Log2 Fold Change", title ="CRC+Fuso vs CRC-Fuso")

plt_MA_Fuso

ggsave("./generated_figures/plt_MA_Fuso.png", plt_MA_Fuso, width = 10, height = 5)

#Plot a volcano 
df_volcano_fuso <- tibble(data.frame(res_Fnucl)) %>%
  mutate("Expression" = case_when(log2FoldChange >= 2 & padj<= 0.05 ~ "Up-regulated in CRC+Fuso",
                                  log2FoldChange <= -2 & padj <= 0.05 ~ "Down-regulated in CRC+Fuso",
                                  TRUE ~ "Unchanged")) %>%
  mutate("ensembl_gene_id"= row.names(res_Fnucl)) %>%
  left_join(coding_genes[, c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") 




plt_volcano_fuso <- ggplot(df_volcano_fuso, aes(x = log2FoldChange, y = -log10(padj), color = Expression))+
  geom_point(alpha = 0.5)+
  geom_hline(yintercept = -log10(0.05), linetype = "dashed")+
  geom_vline(xintercept=c(-2,2), linetype = "dashed")+
  xlim(-7,7)+
  ylim(0,6)+
  scale_color_manual(values = c("Up-regulated in CRC+Fuso" = "#dfb5cc",
                                "Down-regulated in CRC+Fuso" = "#b5a8f1",
                                "Not significant" = "#bebebe"))+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_fuso,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)+
  labs(title = "CRC+Fuso vs CRC-Fuso")

plt_volcano_fuso

#ggsave("./generated_figures/plt_volcano_Fuso.png", plt_volcano_fuso)



#==================================
# Which Pathways are enriched?
#==================================

interesting_genes_FnuclPositive <- row.names(res_Fnucl_sig)[res_Fnucl_sig$log2FoldChange>0]
interesting_genes_FnuclNegative <- row.names(res_Fnucl_sig)[res_Fnucl_sig$log2FoldChange<0] 


##### Run the Enrichment ####################

#Enriched in Fnucl Positive

FnuclPos_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                   background_genes, interesting_genes_FnuclPositive)
 
## convert the list to dataframe
FnuclPos_pathways <- do.call(rbind.data.frame, FnuclPos_pathways)
dim(FnuclPos_pathways) 
 

 
if(length(which(FnuclPos_pathways$p_val < 0.2))>0){
   ##sort by pval
   FnuclPos_pathways <- FnuclPos_pathways[order(FnuclPos_pathways$p_val),]
   ## MHT correction (FDR)
   FnuclPos_pathways$p_adj <- p.adjust(FnuclPos_pathways$p_val, method = "BH")
   
   FnuclPos_pathways <- FnuclPos_pathways[FnuclPos_pathways$p_adj < 0.2,]
   dim(FnuclPos_pathways) 
 }
 

# Enriched in Healthy 
 
FnuclNeg_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                      background_genes, interesting_genes_FnuclNegative)

## convert the list to dataframe
FnuclNeg_pathways <- do.call(rbind.data.frame, FnuclNeg_pathways)
dim(FnuclNeg_pathways) 

##sort by pval
FnuclNeg_pathways <- FnuclNeg_pathways[order(FnuclNeg_pathways$p_val),]
## MHT correction (FDR)
FnuclNeg_pathways$p_adj <- p.adjust(FnuclNeg_pathways$p_val, method = "BH")

length(which(FnuclNeg_pathways$p_adj < 0.2))

FnuclNeg_pathways <- FnuclNeg_pathways[FnuclNeg_pathways$p_adj < 0.2,]
dim(FnuclNeg_pathways) 


# Make a quick plot, but only FusoNegative since enrichment in Fusopositive

if(length(FnuclPos_pathways)>0 & length(FnuclNeg_pathways)>0){
   plt_enrichment_Fuso <- plot_enrichment_results(FnuclPos_pathways, FnuclNeg_pathways, 
                                                  names = c("CRC+Fuso", "CRC-Fuso"))
   plt_enrichment_Fuso
 }


plt_enrichment_FusoNeg<-
ggplot(FnuclNeg_pathways, aes(x = genes_of_interest_in_pathway, y = fct_reorder(pathway, p_adj, .desc = T)))+
  geom_bar(stat = "identity", fill = "#6697ea")+
  labs(y = "Pathway", caption = "*No pathways were significantly enriched in CRC+Fuso",
       title = "Enriched Pathways in CRC-Fuso*", x = "Genes of Interest In Pathway")+
  theme_light()+
  theme(text = element_text(size = 8))

#ggsave("./generated_figures/plt_enrichment_FusoNeg.png", plt_enrichment_FusoNeg, height = 5, width=8, scale = 0.8) 





#=====================================================================================================
# Q3. Find significant DE genes in CRC (+/- *B. fragilis* samples) to the effect of *B. fragilis* on 
#     gene expression
#=====================================================================================================


#These CRC samples were identified (by HeaDs' analysis) as having elevated levels of B. fragilis

high_BFrag <- c("La27",
                "La31",
                "La70",
                "La80",
                "La96",
                "La106",
                "La110")


# Add variable for high F. nucleatum 

coldata_CRC$high_BFrag <- ifelse(row.names(coldata_CRC) %in% high_BFrag, "high_BFrag", "low_BFrag")


# Make and run DESeq

dds_BFrag <-  DESeqDataSetFromMatrix(countData=cts_CRC,
                                     colData=coldata_CRC,
                                     design= ~ high_BFrag)

dds_BFrag <- DESeq(dds_BFrag)

#Define the contrast 
contrast <- c("high_BFrag", c("high_BFrag", "low_BFrag"))

# Generate the res_BFragults
res_BFrag <- results(dds_BFrag, contrast=contrast)
# shrink fold changes for lowly expressed genes
# use ashr so we can use `contrast` as conversion to coef is not trivial
# see https://bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.html #extended-section-on-shrinkage-estimators
#res_BFrag <- lfcShrink(dds_BFrag, contrast=contrast, res_BFrag=res_BFrag, type="ashr")

# MA plot 
plotMA(res_BFrag, ylim=c(-2,2))

# sort by p-value
res_BFrag <- res_BFrag[order(res_BFrag$padj),]
summary(res_BFrag, alpha = 0.05)

# Get significant genes
res_BFrag_sig <- subset(res_BFrag, padj < 0.05 & abs(log2FoldChange) >=2)


#Get top X up and downregulated genes for labelling 

top_genes_Bfrag <- c(row.names(subset(res_BFrag_sig[order(res_BFrag_sig$padj),], log2FoldChange>2))[1:20],
                    row.names(subset(res_BFrag_sig[order(res_BFrag_sig$padj),], log2FoldChange<(-2)))[1:20])

#MA plot 

df_MA_Bfrag <- plotMA(res_BFrag, returnData=T) %>%
  mutate("significant" = case_when(lfc >= 2 & isDE ~ "Up-regulated",
                                   lfc <= -2 & isDE  ~ "Down-regulated",
                                   isDE & lfc<2 & lfc>-2 ~ "Significant, but low LFC",
                                   TRUE ~ "Not significant")) %>%
  mutate("ensembl_gene_id"= row.names(res_BFrag)) %>%
  left_join(coding_genes[, c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") 

plt_MA_Bfrag<-
  ggplot(df_MA_Bfrag, aes(x = log2(mean), y = lfc, color = significant))+
  geom_point( alpha = 0.5)+
  scale_color_manual(values = c("Up-regulated" = "#ebc17c",
                                "Down-regulated" = "#12aa10",
                                "Significant, but low LFC" = "#7f7f7f",
                                "Not significant" = "#bebebe"))+
  ylim(-8,8)+
  geom_hline(yintercept = 2, linetype = "dashed")+
  geom_hline(yintercept = -2, linetype = "dashed")+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_Bfrag,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)+
  labs(x="Log2 Mean Expression", y = "Log2 Fold Change", title ="CRC+Bfrag vs CRC-Bfrag")

plt_MA_Bfrag

ggsave("./generated_figures/plt_MA_Bfrag.png", plt_MA_Bfrag, width = 10, height = 5)

#Plot a volcano 
df_volcano_Bfrag <- tibble(data.frame(res_BFrag)) %>%
  mutate("Expression" = case_when(log2FoldChange >= 2 & padj<= 0.05 ~ "Up-regulated in CRC+Bfrag",
                                  log2FoldChange <= (-2) & padj <= 0.05 ~ "Down-regulated in CRC+Bfrag",
                                  TRUE ~ "Unchanged")) %>%
  mutate("ensembl_gene_id"= row.names(res_BFrag)) %>%
  left_join(coding_genes[, c("ensembl_gene_id", "external_gene_name")], by = "ensembl_gene_id") 



plt_volcano_Bfrag <- ggplot(df_volcano_Bfrag, aes(x = log2FoldChange, y = -log10(padj), color = Expression))+
  geom_point(alpha = 0.5)+
  geom_hline(yintercept = -log10(0.05), linetype = "dashed")+
  geom_vline(xintercept=c(-2,2), linetype = "dashed")+
  scale_color_manual(values = c("Up-regulated in CRC+Bfrag" = "#ebc17c",
                                "Down-regulated in CRC+Bfrag" = "#12aa10",
                                "Unchanged" = "#bebebe"))+
  labs(title = "CRC+Bfrag vs CRC-Bfrag")+
  geom_text_repel(aes(label=ifelse(ensembl_gene_id %in% top_genes_Bfrag,as.character(external_gene_name),'')),
                  max.overlaps = Inf,
                  #nudge_y = 0,
                  color = "black",
                  max.time = 10,
                  force = 6,
                  size = 2)

plt_volcano_Bfrag

#ggsave("./generated_figures/plt_volcano_Bfrag.png", plt_volcano_Bfrag)



#==================================
# Which Pathways are enriched?
#==================================

interesting_genes_BFragPositive <- row.names(res_BFrag_sig)[res_BFrag_sig$log2FoldChange>0]
interesting_genes_BFragNegative <- row.names(res_BFrag_sig)[res_BFrag_sig$log2FoldChange<0] 


##### Run the Enrichment ####################

# Enriched in BFrag Positive

BFragPos_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                       background_genes, interesting_genes_BFragPositive)

## convert the list to dataframe
BFragPos_pathways <- do.call(rbind.data.frame, BFragPos_pathways)
dim(BFragPos_pathways) 

##sort by pval
BFragPos_pathways <- BFragPos_pathways[order(BFragPos_pathways$p_val),]


## MHT correction (FDR)
BFragPos_pathways$p_adj <- p.adjust(BFragPos_pathways$p_val, method = "BH")

BFragPos_pathways <- BFragPos_pathways[BFragPos_pathways$p_adj < 0.2,]
dim(BFragPos_pathways) 




# Enriched in BFrag Negative

BFragNeg_pathways <- msigdb_enrichment(msigdb_C2_CP, msigdb_C2_CP_unique_pathways,
                                       background_genes, interesting_genes_BFragNegative)

## convert the list to dataframe
BFragNeg_pathways <- do.call(rbind.data.frame, BFragNeg_pathways)
dim(BFragNeg_pathways) 

##sort by pval
BFragNeg_pathways <- BFragNeg_pathways[order(BFragNeg_pathways$p_val),]

## MHT correction (FDR)
BFragNeg_pathways$p_adj <- p.adjust(BFragNeg_pathways$p_val, method = "BH")

length(which(BFragNeg_pathways$p_adj < 0.2))

BFragNeg_pathways <- BFragNeg_pathways[BFragNeg_pathways$p_adj < 0.2,]
dim(BFragNeg_pathways) 


# Make a quick plot - Bfrag Enrichment

plt_enrichment_BFrag <- plot_enrichment_results(BFragPos_pathways, BFragNeg_pathways, 
                                              names = c("CRC+BFrag", "CRC-BFrag"))+
  theme(axis.text.y = element_text(size = 5))

plt_enrichment_BFrag


#ggsave("./generated_figures/plt_enrichment_BFrag.png", plt_enrichment_BFrag, height = 5, width = 8)


#=================
# Figures 
#=================

library(egg)

#Standardize variable colors
colors <- c("CRC" = "#d55e00", 
            "Healthy" = "#999999", 
            "Paired" = "#56b4e9",
            "high_Fnucl" = "#cc79a7",
            "low_Fnucl" = "#785ef0",
            "high_BFrag" = "#ebb358",
            "low_BFrag" = "#00a600")

#Standard theme 

colors_cats <- scale_color_manual(values = colors)
              

#Theme

theme <- theme(axis.text=element_text(size=8),
               axis.title=element_text(size=8),
               legend.text = element_text(size=8),
               legend.title = element_text(size=8),
               legend.key.size = unit(10, "pt"),
               plot.margin = margin(10, 0, 10, 0, "pt"))


#Plots 

plt_PCA_full_type <-  
  plotPCA(vst(dds), intgroup = "Type") + 
    xlim(-80,60)+
    ylim(-35,35)+
    colors_cats + 
    labs(color = "Sample Type") +
    theme


plt_PCA_full_flowcell <-  
  plotPCA(vst(dds), intgroup = "raw_counts_M") +
    xlim(-80,60)+
    ylim(-35,35)+
    scale_color_viridis() + 
    labs(color = "Raw Reads (M)") +
    theme

plt_PCA_CRC_fnucl <-  
  plotPCA(vst(dds_Fnucl), intgroup = "high_Fnucl") + 
  xlim(-80,60)+
  ylim(-35,35)+
  colors_cats +
  labs(color = "F_nucl Presence") +
  theme
  
plt_PCA_CRC_flowcell <-  
  plotPCA(vst(dds_Fnucl), intgroup = "raw_counts_M") + 
  xlim(-80,60)+
  ylim(-35,35)+
  scale_color_viridis() + 
  labs(color = "Raw Reads (M)") + 
  theme

plt_PCA_CRC_bfrag <-  
  plotPCA(vst(dds_BFrag), intgroup = "high_BFrag") + 
  xlim(-80,60)+
  ylim(-35,35)+
  colors_cats +
  labs(color = "B_Frag Presence") +
  theme


plts_PCAgrid<-
  list(plt_PCA_full_type,
            plt_PCA_full_flowcell,
            plt_PCA_CRC_fnucl,
            plt_PCA_CRC_flowcell,
            plt_PCA_CRC_bfrag)



plt_PCAgrid <- plot_grid(plotlist = plts_PCAgrid,
                          ncol = 2, nrow = 3, 
                          align = "hv", 
                          labels = c("A", "B", "C", "D", "E"),
                          hjust = -1.7,
                          vjust = 1.3)+
  theme(plot.background = element_rect(fill = "white", colour = NA))
                          

plt_PCAgrid

#save_plot("./generated_figures/plt_PCAgrid.png", plt_PCAgrid, base_asp = 2)
                           
#ggsave("./generated_figures/plt_PCAgrid.png", plt_PCAgrid)

#================
#Export the data 
#================

# DESeq results

write.table(data.frame(res), 
            "./generated_data/DESeq2_CRCvsHealthyOrPaired.tsv",
            col.names = NA, quote = F, sep = "\t")

write.table(data.frame(res_sig), 
            "./generated_data/DESeq2_CRCvsHealthyOrPaired_significant.tsv",
            col.names = NA, quote = F, sep = "\t")


write.table(data.frame(res_Fnucl), 
            "./generated_data/DESeq2_CRCFusoVsCRCnoFuso.tsv",
            col.names = NA, quote = F, sep = "\t")

write.table(data.frame(res_Fnucl_sig), 
            "./generated_data/DESeq2_CRCFusoVsCRCnoFuso_significant.tsv",
            col.names = NA, quote = F, sep = "\t")

write.table(data.frame(res_BFrag), 
            "./generated_data/DESeq2_CRCBFragVsCRCnoBFrag.tsv",
            col.names = NA, quote = F, sep = "\t")

write.table(data.frame(res_BFrag_sig), 
            "./generated_data/DESeq2_CRCBFragVsCRCnoBFrag_significant.tsv",
            col.names = NA, quote = F, sep = "\t")

# Enrichment 

writeLines(interesting_genes_UPcrc, "./generated_data/interesting_genes_UPcrc.txt")
writeLines(interesting_genes_DOWNcrc, "./generated_data/interesting_genes_DOWNcrc.txt")
writeLines(interesting_genes_FnuclPositive, "./generated_data/interesting_genes_FnuclPositive.txt")
writeLines(interesting_genes_FnuclNegative, "./generated_data/interesting_genes_FnuclNegative.txt")
writeLines(interesting_genes_BFragPositive, "./generated_data/interesting_genes_BFragPositive.txt")
writeLines(interesting_genes_BFragNegative, "./generated_data/interesting_genes_BFragNegative.txt")

# Pathway Analysis 


write.table(CRC_pathways, "./generated_data/Enrichment_CRCpathways.txt", 
            col.names = T, row.names = F, quote = F, sep = "\t")
write.table(Healthy_pathways, "./generated_data/Enrichment_Healthypathways.txt", 
            col.names = T, row.names = F, quote = F, sep = "\t")
write.table(FnuclNeg_pathways, "./generated_data/Enrichment_FnuclNegpathways.txt", 
            col.names = T, row.names = F, quote = F, sep = "\t")

write.table(BFragNeg_pathways, "./generated_data/Enrichment_BFragNegpathways.txt", 
            col.names = T, row.names = F, quote = F, sep = "\t")

write.table(BFragPos_pathways, "./generated_data/Enrichment_BFragPospathways.txt", 
            col.names = T, row.names = F, quote = F, sep = "\t")

#Gene information 

write.table(coding_genes, "./generated_data/gene_annotation_info.txt",
            col.names = T, row.names = F, quote = F, sep = "\t")


