library(tidyverse)
library(BSDA)
library(phyloseq)
library(DESeq2)
library(pheatmap)


#=====================================
# Microbiome Analysis 
#=====================================

#=======================
# Read in the metadata
#=======================

metadata <- read.delim("./metadata/metadata_Rformatted.tsv")

# Extract Patient_ID 
metadata <- 
  metadata %>% 
  separate(pt_ID, sep = "-", into = c("Patient_ID")) #Get patient ID column 

metadata_subset <- subset(metadata, Type != "Control" & (Patient_ID!="030"|Patient_ID!="055")) 
#Exclude the control for now. Also P030 and P055 because they weren't part of the study 

#======================================
# Read in the summarized bracken data
#======================================

data_bracken <- read.delim("./generated_data/bracken/bac_abundance_table.csv",
                           row.names = 1)

data_bracken_counts <- read.delim("./generated_data/bracken/bac_count_table.csv",
                                  row.names = 1)

#=======================================
# Read in Kraken Data
#=======================================

kraken_dir <- "./generated_data/kraken/"

kraken_data <- 
  list.files(kraken_dir, full.names = T) %>%
  set_names(., nm = map(.x = ., ~gsub(".kraken", "", basename(.x)))) %>%
  map(function(x) {
    x <- read.delim(x, header = F)
    x$V6 <- trimws(x$V6, which = "left")
    x <- subset(x, V6 %in% c("root", "Bacteria", "Homo sapiens"))
    return(x)}) %>%
  map_df(~as.data.frame(.x), .id = "Sample_ID") %>%
  select(Sample_ID, "Percent_assigned" = V1, 
         "Total_assigned" = V2, "Organism" = V6) %>%
  group_by(Sample_ID) %>%
  summarise(
    "Total_reads" = Total_assigned[Organism=="root"],
    "Bac_reads" = Total_assigned[Organism=="Bacteria"],
    "Human_reads" = Total_assigned[Organism=="Homo sapiens"],
    "Bac_prcnt" = Percent_assigned[Organism=="Bacteria"],
    "Human_prcnt" = Percent_assigned[Organism=="Homo sapiens"])

kraken_data$Sample_ID <- gsub(".out", "", kraken_data$Sample_ID)

kraken_data <- 
  left_join(metadata_subset, kraken_data,  by = c("RNA_seq_ID"="Sample_ID"))

#=====================================
# Stat tests for bacterial percentage
#=====================================

# Is there a difference in bacterial load between Healthy and CRC?
kraken_data_noPaired <- subset(kraken_data, Type != "Paired")
kruskal.test(kraken_data_noPaired$Bac_prcnt, kraken_data_noPaired$Type)


# Is there a difference between bacterial load in Paired and Healthy 
# from the same sample?

diff_BacPrnt_CRCminusPaired <- 
  kraken_data %>%
  filter(Type != "Healthy") %>%
  filter(Patient_ID !="054") %>% #Patient 54 missing paired sample
  group_by(Patient_ID) %>% 
  summarize(diff = diff(Bac_prcnt)) #diff is CRC - Paired

SIGN.test(diff_BacPrnt_CRCminusPaired$diff)

# Sign test was performed because the difference of CRC-Paired is not symmetrical 
# around the median, which is required for wilcox.test 


#==========================================
# Bacterial Community Composition Analysis 
#==========================================


# Create Phyloseq Objects ==========

#OTU_table
otu <- otu_table(data_bracken_counts, taxa_are_rows = TRUE)

#Tax_table
taxa <- 
data.frame(row.names(data_bracken_counts)) %>%
  separate(col = 1, sep = " ", into = c("Genus", "Species", "Strain"), extra = "merge")
taxa$Species <- paste(taxa$Genus, taxa$Species, sep = " ")
row.names(taxa) <- row.names(data_bracken_counts)
taxa <- tax_table(as.matrix(taxa))

#metadata
phylo_metadata <- sample_data(column_to_rownames(metadata, var = "RNA_seq_ID"))

#Phyloseq Object 
phylo_obj <- phyloseq(otu, taxa, phylo_metadata)

#Filter Samples 
phylo_obj_filter <- subset_samples(phylo_obj, Type != "control" & 
                                    (Patient_ID != "030" | Patient_ID != "055"))

# Relative abundance ===================================


#Filter low abundance taxa =====

phylo_obj_relAbund  <- transform_sample_counts(phylo_obj_filter, function(x) x / sum(x) )
phylo_obj_relAbund_Filter <- filter_taxa(phylo_obj_relAbund , function(x) mean(x) > 1e-3, TRUE)


#Subset only Bacteriodes and Fusobacterium
phylo_obj_relAbund_BacterioFusobact <- subset_taxa(phylo_obj_relAbund_Filter, Species=="Bacteroides fragilis"| Species=="Fusobacterium nucleatum")


# Alpha Diversity ==========================

# Compare richness between CRC and Healthy

richness_data_noPaired <- 
  estimate_richness(phylo_obj_filter) %>%
    rownames_to_column() %>%
    left_join(metadata, by = c("rowname" = "RNA_seq_ID")) %>%
    pivot_longer(cols = c("Observed" ,"Chao1" ,"se.chao1" ,"ACE","se.ACE" ,
                          "Shannon" ,"Simpson" ,"InvSimpson"  ,"Fisher")) %>%
    filter(name %in% c("Observed", "Shannon", "Simpson", "Fisher") & Type != "Paired")
  

qqnorm(richness_data_noPaired$value[richness_data_noPaired$name=="Shannon"])
qqnorm(richness_data_noPaired$value[richness_data_noPaired$name=="Observed"]) 


t.test(richness_data_noPaired$value[richness_data_noPaired$name=="Shannon"&richness_data_noPaired$Type=="CRC"],
       richness_data_noPaired$value[richness_data_noPaired$name=="Shannon"&richness_data_noPaired$Type=="Healthy"])

t.test(richness_data_noPaired$value[richness_data_noPaired$name=="Observed"&richness_data_noPaired$Type=="CRC"],
       richness_data_noPaired$value[richness_data_noPaired$name=="Observed"&richness_data_noPaired$Type=="Healthy"])


# Compare richness between CRC and Paired

richness_data_noHealthy <-  
  estimate_richness(phylo_obj_filter) %>%
  rownames_to_column() %>%
  left_join(metadata, by = c("rowname" = "RNA_seq_ID")) %>%
  pivot_longer(cols = c("Observed" ,"Chao1" ,"se.chao1" ,"ACE","se.ACE" ,
                        "Shannon" ,"Simpson" ,"InvSimpson"  ,"Fisher")) %>%
  filter(name %in% c("Observed", "Shannon", "Simpson", "Fisher") & Type != "Healthy")


ggplot(richness_data_noHealthy, aes(x= Type, y = value))+
  facet_wrap(name ~ ., scales = "free", ncol = 2)+
  geom_point()+
  geom_line(aes(group=Patient_ID))

diff_Observed <- 
  richness_data_noHealthy %>%
  filter(name=="Observed") %>%
  group_by(Patient_ID) %>%
  summarize(diff = diff(value)) #Diff is CRC-Paired

qqnorm(diff_Observed$diff)
t.test(diff_Observed$diff, mu = 0)

#Significant for Observed species 

diff_Shannon <- 
  richness_data_noHealthy %>%
  filter(name=="Shannon") %>%
  group_by(Patient_ID) %>%
  summarize(diff = diff(value)) #Diff is CRC-Paired

qqnorm(diff_Shannon$diff)
t.test(diff_Shannon$diff, mu = 0)



#Get abundance of B frag and F nucleatum

abund_BFragFnucleatum<-
  data.frame(otu_table(phylo_obj_relAbund_BacterioFusobact)) %>%
  rownames_to_column(var="species") %>%
  pivot_longer(cols = colnames(otu_table(phylo_obj_relAbund_BacterioFusobact))) %>%
  left_join(metadata, by = c("name" = "RNA_seq_ID"))

# Test specific relative abundance for F. nucleatum and B. fragilis 
# between Healthy/Paired and CRC

# Healthy vs CRC

abund_BFragFnucleatum_noPaired<-
data.frame(otu_table(phylo_obj_relAbund_BacterioFusobact)) %>%
  rownames_to_column(var="species") %>%
  pivot_longer(cols = colnames(otu_table(phylo_obj_relAbund_BacterioFusobact))) %>%
  left_join(metadata, by = c("name" = "RNA_seq_ID")) %>%
  filter(Type != "Paired")


qqnorm(log(abund_BFragFnucleatum_noPaired$value))

t.test(abund_BFragFnucleatum_noPaired$value[abund_BFragFnucleatum_noPaired$species=="Bacteroides fragilis"& abund_BFragFnucleatum_noPaired$Type=="CRC"],
       abund_BFragFnucleatum_noPaired$value[abund_BFragFnucleatum_noPaired$species=="Bacteroides fragilis"& abund_BFragFnucleatum_noPaired$Type=="Healthy"])

t.test(abund_BFragFnucleatum_noPaired$value[abund_BFragFnucleatum_noPaired$species=="Fusobacterium nucleatum"& abund_BFragFnucleatum_noPaired$Type=="CRC"],
       abund_BFragFnucleatum_noPaired$value[abund_BFragFnucleatum_noPaired$species=="Fusobacterium nucleatum"& abund_BFragFnucleatum_noPaired$Type=="Healthy"])

# Paired vs CRC 

abund_BFragFnucleatum_noHealthy<-
  data.frame(otu_table(phylo_obj_relAbund_BacterioFusobact)) %>%
  rownames_to_column(var="species") %>%
  pivot_longer(cols = colnames(otu_table(phylo_obj_relAbund_BacterioFusobact))) %>%
  left_join(metadata, by = c("name" = "RNA_seq_ID")) %>%
  filter(Type != "Healthy")


abund_BFragFnucleatum_noHealthy_diff <- 
  abund_BFragFnucleatum_noHealthy %>%
  group_by(Patient_ID, species) %>%
  summarize(diff = diff(value))

qqnorm(abund_BFragFnucleatum_noHealthy_diff$diff[abund_BFragFnucleatum_noHealthy_diff$species=="Bacteroides fragilis"])
qqnorm(abund_BFragFnucleatum_noHealthy_diff$diff[abund_BFragFnucleatum_noHealthy_diff$species=="Fusobacterium nucleatum"])

SIGN.test(abund_BFragFnucleatum_noHealthy_diff$diff[abund_BFragFnucleatum_noHealthy_diff$species=="Bacteroides fragilis"])
SIGN.test(abund_BFragFnucleatum_noHealthy_diff$diff[abund_BFragFnucleatum_noHealthy_diff$species=="Fusobacterium nucleatum"])


#==================
# Plots 
#==================

# Percentage of Reads Classified to Bacteria 
plt1<-
ggplot(kraken_data, aes(x = Type, y = Bac_prcnt))+
  geom_jitter(width = 0.1)+
  labs(x = "Tissue Type", y = "Percentage of Reads Classified to Bacteria")

# Percentage of Reads Classified to Human 
plt2<-
ggplot(kraken_data, aes(x = Type, y = Human_prcnt))+
  geom_jitter(width = 0.07)+
  geom_boxplot(width=0.3, alpha = 0.5, outlier.shape = NA)+
  labs(x = "Tissue Type", y = "Percentage of Reads Classified as Human")


# Percentage of Reads Classified to Bacteria with Pairedness
plt3<-
ggplot(subset(kraken_data, Type!="Healthy"), aes(x = Type, y = Bac_prcnt))+
  geom_point()+
  geom_line(aes(group = Patient_ID))+
  labs(x = "Tissue Type", y = "Percentage of Reads Classified to Bacteria")


# Bar Plot of relative abundance and Genus 
plt4<-
plot_bar(phylo_obj_relAbund_Filter, x = "Patient_ID", fill = "Genus")+
  facet_grid(~Type, scales = "free_x")+
  labs(y = "Relative Abundance (%)")

# Bar Plot of relative abundance for Fusobacterium and Bacteriodes 
plt5<-
plot_bar(phylo_obj_relAbund_BacterioFusobact, x = "Patient_ID", fill = "Species")+
  facet_grid(~Type, scales = "free_x")+
  labs(y = "Relative Abundance (%)")

plt6<-
#Plot of Alpha Diversity 
plot_richness(phylo_obj_filter, x = "Type", measures = c("Observed", "Shannon"))+
  geom_violin(aes(fill = Type))+
  geom_boxplot(width = 0.1, alpha = 0.5)

#Select Richness Measures - CRC vs Healthy
plt7<-
ggplot(richness_data_noPaired, aes(x= Type, y = value))+
  geom_violin(width = 0.5, aes (fill=Type))+
  geom_jitter(alpha = 0.5, width = 0.05) +
  geom_boxplot(width = 0.08, alpha = 0.5)+
  facet_wrap(name ~ ., scales = "free", ncol = 2)

#Relative abundance of B Frag and F nucleatum all samples
plt8<-
ggplot(abund_BFragFnucleatum, aes(x = Type, y = log(value)))+
  geom_jitter(width = 0.1)+
  geom_boxplot(alpha=0.5)+
  facet_grid(~species)+
  labs(y="log-transformed Relative Abundance")

#Paired Relative abundance of B Frag and F nucleatum 
plt9<-
ggplot(subset(abund_BFragFnucleatum, Type!="Healthy"), aes(x = Type, y = log(value)))+
  geom_point()+
  geom_line(aes(group=Patient_ID))+
  facet_grid(~species)


#===================================
# Analysis of Fap2, fadA, BFT 
#===================================

#Read in the data 
virulence_counts <- read.delim("./generated_data/virulence_factors/count_matrix_unmapped.tsv")
number_of_reads <- read.delim("./generated_data/virulence_factors/multiqc_featureCounts.txt")

number_of_reads$Sample <- gsub("_alignUnmapped", "", number_of_reads$Sample)

#Join with metadata 
virulence_counts2<-
virulence_counts %>%
  pivot_longer(cols = 2:119) %>%
  left_join(kraken_data, by = c("name"="RNA_seq_ID")) %>%
  left_join(abund_BFragFnucleatum[,1:3], by = "name") 
                    

#Unable to find a signal for BFT 

# Looking closet at Fap2, fadA data 

data_vir_Fuso <- 
  virulence_counts2 %>%
    left_join(number_of_reads, by = c("name" = "Sample")) %>%
   #filter(species == "Fusobacterium nucleatum") %>%
    mutate("prcnt_vir_counts" = value.x/Total)

ggplot(subset(data_vir_Fuso, species != "NA"), aes(x=prcnt_vir_counts, y = value.y)) +
  geom_point()+
  facet_wrap(~species+gene, ncol = 3)+
  geom_smooth(method='lm', formula= y~x)









