library(tidyverse)
library(ggplot2)
library(ggpubr)
library(ggrepel)
#library(ggforce)
library(taxize)
library(rstatix)

# counts of phyla: Do we observe a difference in the abundance of the bacterial phyla between sample groups?
# done in a similar way to how we generated the counts per kingdom.

#=======================
# Read in the updated metadata and tax table
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

#this is where the bracken counts files are
dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
plot_dir <- "/media/henrike/E586-BCAE/Data_for_HeaDS/figures_HZ/"

tax_all  <- read.csv(paste0(dir_dat, 'tax_table.csv'), sep = '\t')

#which phyla exist?
tax_all %>%
  distinct(phylum)

#there are 64 phyla in total

bact_phyla <- tax_all %>%
  filter(superkingdom == 'Bacteria') %>%
  distinct(phylum)
bact_phyla <- unlist(bact_phyla)
bact_phyla[is.na(bact_phyla)] <- 'NA'
#38 are bacterial phyla

tax_all %>%
  filter(superkingdom == 'Archaea') %>%
  distinct(phylum)
#7 are archaea

tax_all %>%
  filter(superkingdom == 'Viruses') %>%
  distinct(phylum)
# 18 are viruses

#chordata is the phylum of humans

bact_phyla <- tax_all %>%
  filter(superkingdom == 'Bacteria') %>%
  distinct(phylum)

#=======================
# Loop over all count files
#=======================

phylum_cts <- data.frame()

for (i in seq(4,124,1)) {
  
  if(!(i %in% c(45,71,72))){
    
    print(i)
    
    bracken_path <- paste0(dir_dat,"La", i, '.non_0_scaled.out')
    cts <- as_tibble(read.csv(bracken_path, sep = '\t'))
    
    sum_phyl <- cts %>%
      filter(superkingdom == 'Bacteria') %>%
      group_by(phylum) %>%
      summarise(scaled_sum = sum(scaled_cts)) %>% 
      pivot_wider(names_from = phylum, values_from = scaled_sum) %>%
      mutate(sample = paste0('La',i)) %>% #add sample
      relocate(sample) #put the sample col to the front
    
    #add 0 counts for not observed phyla
    missing <- bact_phyla[!bact_phyla %in% colnames(sum_phyl)]
    zero_vec <- as.data.frame(t(rep(0,length(missing))))
    colnames(zero_vec) <- missing
    
    sum_phyl <- cbind(sum_phyl,zero_vec)
    
    #add to the kingdom_cts
    phylum_cts <- rbind(phylum_cts, sum_phyl)
    
  }
}

#merge the metadata to it
phylum_cts_m <- merge(phylum_cts, metadata, by = 'sample')

write.table(format(phylum_cts_m, digits = 3, scientific = F), paste0(dir_dat, 'phylum_counts.csv'), quote = F, 
            row.names = F, sep = '\t')

#=======================
# Read in existing phylum data:
#=======================

#read it in again
phylum_cts_m <- read.csv(paste0(dir_dat, 'phylum_counts.csv'), sep = '\t')

#=======================
# Test everything!
#=======================

pvals <- c()

#ind tests

pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Actinobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Proteobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Armatimonadetes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Bacteroidota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Firmicutes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Thermotogae ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Chlorobi ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Cyanobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Verrucomicrobia ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Synergistetes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Fusobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Chloroflexi ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Deinococcus.Thermus ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Spirochaetes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Ignavibacteriae ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Planctomycetota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Tenericutes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Deferribacteres ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Thermodesulfobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Kiritimatiellaeota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Nitrospirae ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Candidatus.Saccharibacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Chlamydiae ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Aquificae ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Gemmatimonadetes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Dictyoglomi ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Elusimicrobia ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Balneolaeota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Acidobacteria ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Calditrichaeota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Candidatus.Omnitrophica ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Candidatus.Bipolaricaulota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Coprothermobacterota ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Candidatus.Cloacimonetes ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Caldiserica ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Fibrobacteres ~ type) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Healthy')) %>% wilcox_test(Chrysiogenetes ~ type) %>% select(p))

#paired tests

pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Actinobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Proteobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Armatimonadetes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Bacteroidota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Firmicutes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Thermotogae ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Chlorobi ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Cyanobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Verrucomicrobia ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Synergistetes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Fusobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Chloroflexi ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Deinococcus.Thermus ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Spirochaetes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Ignavibacteriae ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Planctomycetota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Tenericutes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Deferribacteres ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Thermodesulfobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Kiritimatiellaeota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Nitrospirae ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Candidatus.Saccharibacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Chlamydiae ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Aquificae ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Gemmatimonadetes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Dictyoglomi ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Elusimicrobia ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Balneolaeota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Acidobacteria ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Calditrichaeota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Candidatus.Omnitrophica ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Candidatus.Bipolaricaulota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Coprothermobacterota ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Candidatus.Cloacimonetes ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Caldiserica ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Fibrobacteres ~ type, paired = T) %>% select(p))
pvals <- append(pvals, phylum_cts_m %>% filter(type %in% c('CRC', 'Paired')) %>% filter(patient_ID != 54) %>% wilcox_test(Chrysiogenetes ~ type, paired = T) %>% select(p))

names(pvals) <- c(paste0('ind.',bact_phyla[bact_phyla != 'NA']), paste0('pair.',bact_phyla[bact_phyla != 'NA']))
pvals[pvals < 0.05]
pvals <- p.adjust(pvals)

#we find the following significant p-values after multiple testing correction:
pvals[pvals < 0.05]

# Some violin plots of the two phyla:

p_syn <- ggplot(phylum_cts_m, aes(y=Synergistetes, x = type)) +
  geom_violin() +
  ggtitle('Counts of Synergistetes') +
  geom_jitter(position = position_jitter(0.1))  +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'Synergistetes_cts.png'), width = 800, height = 600)
p_syn
dev.off()

#La110 has a crazy high count
phylum_cts_m %>%
  select(sample, Synergistetes, type) %>%
  arrange(desc(Synergistetes)) %>%
  head()

p_syn <- ggplot(phylum_cts_m, aes(y=Synergistetes, x = type)) +
  geom_violin() +
  ggtitle('Counts of Synergistetes, Zoom') +
  geom_jitter(position = position_jitter(0.1)) +
  ylim(0,750) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'Synergistetes_cts_zoom.png'), width = 800, height = 600)
p_syn
dev.off()

p_fus <- ggplot(phylum_cts_m, aes(y=Fusobacteria, x = type)) +
  geom_violin() +
  ggtitle('Counts of Fusobacteria') +
  geom_jitter(position = position_jitter(0.1))  +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'Fusobacteria_cts.png'), width = 800, height = 600)
p_fus
dev.off()

phylum_cts_m %>%
  select(sample, Fusobacteria, type) %>%
  arrange(desc(Fusobacteria)) %>%
  head(n=20)


p_fus <- ggplot(phylum_cts_m, aes(y=Fusobacteria, x = type)) +
  geom_violin() +
  ggtitle('Counts of Fusobacteria, Zoom') +
  geom_jitter(position = position_jitter(0.1)) +
  ylim(0,4000) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'Fusobacteria_cts_zoom.png'), width = 800, height = 600)
p_fus
dev.off()

