library(tidyverse)
library(ggplot2)
library(ggrepel)
library(reshape)
library(phyloseq)
library(rstatix)
library(ggpubr)

#=======================
# Read in the metadata
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

#this is the lowest amount of counts (approximator for amount of reads) across all files
min_cts <- min(metadata$tot_0)

dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
plot_dir <- "/media/henrike/E586-BCAE/Data_for_HeaDS/figures_HZ/"

theme_set(theme_bw(base_size = 22))

color_dict <- c('CRC' = '#D55E00', 'Healthy' = "#999999", 'Paired' = "#56B4E9")

#=======================
# Phyloseq
#=======================

# for richness estimation scaled counts cannot be used according to: 
# https://github.com/joey711/phyloseq/issues/287

# So we load in the filtered table of the orig counts
OTU_count_table_filtered <- read.csv(paste0(dir_dat, 'OTU_count_table_filtered.out'), sep = '\t') 

#omit human since we're look at bacterial composition
OTU_count_table_filtered$taxID <- rownames(OTU_count_table_filtered)
OTU_count_table_filtered <- OTU_count_table_filtered %>%
  filter(taxID != '9606') %>%
  select(-taxID)

# now load in the tax table and make phyloseq object
# according to documentation it should be a matrix with the relevant row and column names:
# https://joey711.github.io/phyloseq/import-data.html
a <- read.csv(paste0(dir_dat, 'tax_table.csv'), sep = '\t')
b <- a %>%
  select(-taxID)
c <- as.matrix(b)
rownames(c) <- paste0('taxID.',a$taxID)
colnames(c)
TAX = tax_table(c)
#check
TAX

#perhaps having numbers as taxa distrubs phyloseq. Lets put latters in from of them:
rownames(OTU_count_table_filtered) <- paste0('taxID.', rownames(OTU_count_table_filtered))

#perhaps having NAs disturbs phyloseq. Lets replace them with 0's
OTU_count_table_filtered[is.na(OTU_count_table_filtered)] <- 0

#make phyloseq OTU table
OTU_f = otu_table(OTU_count_table_filtered, taxa_are_rows = T)

#metadata
phylo_metadata <- sample_data(column_to_rownames(metadata, var = "sample"))

physeq_obj_f = phyloseq(OTU_f, TAX, phylo_metadata)
physeq_obj_f


#=======================
# some plots
#=======================

#plot_bar(physeq_obj_f, fill = "phylum")

# there might be diff in the dominating phylum/phyla between samples.
# get phylum counts (like the superkingdom counts) 

#=======================
# alpha diversity
#=======================

rich_filtered <- estimate_richness(physeq_obj_f) %>%
  rownames_to_column() %>%
  left_join(metadata, by = c("rowname" = "sample"))


#=======================
# some plots
#=======================

# Shannon VS library size
# Looks the same as in Blaine's analysis

p2 <- ggplot(rich_filtered, aes(x=Shannon, y=log2(tot_0), color = type)) +
  ggtitle('Filtered for detection threshold') +
  scale_color_manual(values=color_dict) +
  geom_point(size = 3) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'Shannon_VS_lib_depth_filtered.png'), width = 1000, height = 600)
p2
dev.off()

# violin plots of the different diversity measures

p1 <- ggplot(rich_filtered, aes(y=Shannon, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)
  
png(filename = paste0(plot_dir, 'violin_Shannon.png'), width = 800, height = 600)
p1
dev.off()

# Chao1
p1 <- ggplot(rich_filtered, aes(y=Chao1, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'violin_Chao1.png'), width = 800, height = 600)
p1
dev.off()

# ACE
p1 <- ggplot(rich_filtered, aes(y=ACE, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'violin_ACE.png'), width = 800, height = 600)
p1
dev.off()

# Simpson
p1 <- ggplot(rich_filtered, aes(y=Simpson, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'violin_Simpson.png'), width = 800, height = 600)
p1
dev.off()

# InvSimpson
p1 <- ggplot(rich_filtered, aes(y=InvSimpson, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'violin_InvSimpson.png'), width = 800, height = 600)
p1
dev.off()

# Fisher
p1 <- ggplot(rich_filtered, aes(y=Fisher, x = type)) +
  geom_violin() +
  geom_jitter(position = position_jitter(0.1)) +
  theme_bw(base_size = 22)

png(filename = paste0(plot_dir, 'violin_Fisher.png'), width = 800, height = 600)
p1
dev.off()

#=======================
# check normality/ qq plots
#=======================

p1 <- ggqqplot(rich_filtered, x = "Shannon", facet.by = "type", main = 'Shannon')

png(filename = paste0(plot_dir, 'qq_Shannon.png'), width = 800, height = 600)
p1
dev.off()

#outliers
rich_filtered %>%
  group_by(type) %>%
  identify_outliers(Shannon) %>%
  relocate(is.outlier, is.extreme)

#test for normality
rich_filtered %>%
  group_by(type) %>%
  shapiro_test(Shannon)

#test for equality of variance
rich_filtered %>%
  levene_test(Shannon ~ type)

#=======================
# t-tests
#=======================

# for CRC VS Healthy and Healthy VS Paired we need an independent test
# for CRC VS Paired we can do a paired t-test

#one sided t-test for Shannon index being greater in CRC 
crc_VS_healthy <- rich_filtered %>% 
  filter(type %in% c('CRC','Healthy')) %>%
  t_test(Shannon ~ type, alternative = 'greater') %>%
  add_significance()
crc_VS_healthy

paired_VS_healthy <- rich_filtered %>% 
  filter(type %in% c('Paired','Healthy')) %>%
  t_test(Shannon ~ type, alternative = 'greater') %>%
  add_significance()
paired_VS_healthy

crc_VS_paired <- rich_filtered %>%
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  t_test(Shannon ~ type, alternative = 'greater', paired = T) %>%
  add_significance()
crc_VS_paired

#correct p-values for multiple testing since I did the tests separately:
pvals <- c(crc_VS_healthy$p, paired_VS_healthy$p, crc_VS_paired$p)
pvals <- p.adjust(pvals)
names(pvals) <- c('ind.CRC.Healthy', 'ind.Healthy.Paired', 'pair.CRC.Paired')
pvals

#correct p-values for multiple testing since I did the tests separately:
pvals <- c(crc_VS_healthy$p, crc_VS_paired$p)
pvals <- p.adjust(pvals)
names(pvals) <- c('ind.CRC.Healthy','pair.CRC.Paired')
pvals

# add tests for the number of observed species, though this is highly dependent on sequencing depth 
# and not a good measure of diversity!
crc_VS_healthy_obs <- rich_filtered %>% 
  filter(type %in% c('CRC','Healthy')) %>%
  t_test(Observed ~ type, alternative = 'greater') %>%
  add_significance()

paired_VS_healthy_obs <- rich_filtered %>% 
  filter(type %in% c('Paired','Healthy')) %>%
  t_test(Observed ~ type, alternative = 'greater') %>%
  add_significance()

crc_VS_paired_obs <- rich_filtered %>%
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  t_test(Observed ~ type, alternative = 'greater', paired = T) %>%
  add_significance()

#correct p-values for multiple testing since I did the tests separately:
all_pvals <- c(crc_VS_healthy$p, paired_VS_healthy$p, crc_VS_paired$p, 
               crc_VS_healthy_obs$p, paired_VS_healthy_obs$p, crc_VS_paired_obs$p)
all_pvals <- p.adjust(all_pvals)
names(all_pvals) <- c('ind.Shannon.CRC.Healthy', 'ind.Shannon.Healthy.Paired', 'pair.Shannon.CRC.Paired',
                      'ind.Obs.CRC.Healthy', 'ind.Obs.Healthy.Paired', 'pair.Obs.CRC.Paired')
all_pvals
