library(tidyverse)
library(ggplot2)
library(ggpubr)
library(ggrepel)
library(ggforce)
library(rstatix)

# Which fraction of reads come from bacteria?

# We will need to identify the sample with the fewest reads and down-sample all other
# count tables to that, as shown is subsample_count_matrix.R . 
# Then we identify the superkingdom of each species

#set ggplot theme
theme_set(theme_bw(base_size = 22))

#=======================
# Read in the updated metadata
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

#this is the lowest amount of counts (approximator for amount of reads) across all files
min_cts <- min(metadata$tot_0)

#this is where the bracken counts files are
dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
plot_dir <- "/media/henrike/E586-BCAE/Data_for_HeaDS/figures_HZ/"

##############################################

#=======================
#load in the tax string table
#=======================

tax_all  <- read.csv(paste0(dir_dat, 'tax_table.csv'), sep = '\t')

#=======================
#go through all of the count files
#=======================

kingdom_cts <- data.frame(Archaea = numeric(),
                          Bacteria = numeric(),
                          Eukaryota = numeric(),
                          Viruses = numeric(),
                          None = numeric(),
                          Human = numeric())

# go through all samples
for (i in seq(4,124,1)) {
  
  if(!(i %in% c(45,71,72))){
    
    print(i)
    
    bracken_path <- paste0(dir_dat,"La", i, '.bracken.out')
    bracken_raw <- as_tibble(read.csv(bracken_path, sep = '\t'))
    
    bracken_gt0 <- bracken_raw %>%
      filter(new_est_reads > 0)
    
    #merge the tax info to the counts
    bracken_gt0_tax <- merge(bracken_gt0, tax_all, by.x = 'taxonomy_id', by.y = 'taxID')
    
    #now we scale
    bracken_gt0_tax <- bracken_gt0_tax %>%
      mutate(scaled_cts = new_est_reads*metadata[metadata$sample == paste0('La',i),c('scaling')])
    
    #sort again by assigned reads (scaled one this time)
    bracken_gt0_tax <- bracken_gt0_tax %>% arrange(desc(scaled_cts))
    
    #move scaled_cts to after new_est_reads
    bracken_gt0_tax <- bracken_gt0_tax %>% relocate(scaled_cts, .after = new_est_reads)
    
    write.table(format(bracken_gt0_tax, digits = 3, scientific = F), 
                paste0(dir_dat, 'La', i, '.non_0_scaled.out'), quote = F, 
                row.names = F, sep = '\t')
    
    #now, we can calc the sum of counts for each superkingdom
    sum_king <- bracken_gt0_tax %>%
      group_by(superkingdom) %>%
      summarise(scaled_sum = sum(scaled_cts))

    #transpose
    sum_king_t <- data.frame(t(sum_king[-1]))
    colnames(sum_king_t) <- sum_king$superkingdom

    #fix NA (blank) column name that sometimes appears with plasmids (they don't have a superkingdom)
    if(any(is.na(colnames(sum_king_t)))){
      colnames(sum_king_t)[is.na(colnames(sum_king_t))] <- 'None'
    } else{
      sum_king_t$None <- 0
    }

    #add human cts
    sum_king_t$Human <- bracken_gt0_tax %>%
      filter(name == 'Homo sapiens') %>%
      summarise(scaled_sum = sum(scaled_cts)) %>%
      as.numeric()

    #set the row name to the sample ID
    rownames(sum_king_t) <- paste0('La', i)

    #add to the kingdom_cts
    kingdom_cts <- rbind(kingdom_cts, sum_king_t)
    
  }
  
}

#add the sample name as a column and merge the metadata to the kingdom counts
kingdom_cts$sample <- rownames(kingdom_cts)

kingdom_cts_m <- merge(kingdom_cts, metadata, by = 'sample')

write.table(format(kingdom_cts_m, digits = 3, scientific = F), paste0(dir_dat, 'kingdom_counts.csv'), quote = F, 
          row.names = F, sep = '\t')

#load in again:
kingdom_cts_m <- read.csv(paste0(dir_dat, 'kingdom_counts.csv'),sep = '\t')

#=======================
# check normality/ qq plots
#=======================

# check distribution of bacterial counts against a normal distr.
# bacterial counts in Healthy and Paired are approx normally distributed, but CRC is very much not.
# This is probably due to the stratification, i.e. there being a subset of CRC 
# samples with very high bact load
png(filename = paste0(plot_dir, 'qq_bacteria_cts.png'), width = 800, height = 600)
ggqqplot(kingdom_cts_m, x = "Bacteria", facet.by = "type", main = 'Bacteria')
dev.off()

#how about the other superkingdoms?
ggqqplot(kingdom_cts_m, x = "Archaea", facet.by = "type", main = 'Archaea')
ggqqplot(kingdom_cts_m, x = "Eukaryota", facet.by = "type", main = 'Eukaryota')
ggqqplot(kingdom_cts_m, x = "Viruses", facet.by = "type", main = 'Viruses')
ggqqplot(kingdom_cts_m, x = "None", facet.by = "type", main = 'Plasmids')

#some boxplots/violine plots
p <- ggplot(kingdom_cts_m, aes(y = Bacteria, x = type)) +
  ggtitle('Scaled Counts assigned to Bacteria superkingdom') +
  geom_violin() +
  geom_boxplot(width = 0.1, color = 'grey40', outlier.shape = NA) +
  geom_sina(color = 'grey30', size = 1.5)

png(filename = paste0(plot_dir, 'violin_bacteria_cts.png'), width = 800, height = 600)
p
dev.off()

#outliers
kingdom_cts_m %>%
  group_by(type) %>%
  identify_outliers(Bacteria)

#We also see a couple of outliers, some of which are extreme (CRC): above Q3 + 3xIQR or below Q1 - 3xIQR

##
#side track: are outliers by themselves normally distributed? I.e. are they just from a diff distr?
##

ols <- kingdom_cts_m %>%
  group_by(type) %>%
  identify_outliers(Bacteria) %>%
  filter(is.extreme)

ggqqplot(ols, x = "Bacteria")

#maybe. they are too few to say.<s

#Shapiro wilk test for normality

kingdom_cts_m %>%
  group_by(type) %>%
  shapiro_test(Bacteria)

# The variance also differs between groups, so at the minimum Welchs t-test (not student's t-test).
kingdom_cts_m %>%
  levene_test(Bacteria ~ type)

#However should probably switch to a Wilcoxon test which is more powerful on non-normal data

#=======================
# Wilcoxon tests
#=======================

# Wilcoxon rank sum test: independent samples. One-sided
w.ind <- kingdom_cts_m %>% 
  filter(type %in% c('Healthy', 'CRC')) %>%
  wilcox_test(Bacteria ~ type, alternative = 'greater')

# between CRC and paired we should do a paired Wilcoxon signed rank test
# for the paired test we can only use the samples from the same patient (CRC and paired)

#find out which patients do not have two samples:
kingdom_cts_m %>%
  filter(type != 'Healthy') %>%
  count(patient_ID) %>%
  filter(n == 1)

#patient 54 is missing the Paired sample. Do the paired test without them

w.paired <- kingdom_cts_m %>%
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  wilcox_test(Bacteria ~ type, paired = T, alternative = 'greater')

#=======================
# add multiple testing correction
#=======================

pvals <- w.ind$p
pvals <- append(pvals, w.paired$p)
pvals <- p.adjust(pvals)
names(pvals) <- c('bact.ind.CRC.Healthy', 'bact.pair.CRC.Paired')
pvals  
  
#=======================
# add tests for viruses and archaea
#=======================
  
#test for difference in viral load: CRC vs healthy
vir.ind <- kingdom_cts_m %>% 
  filter(type %in% c('Healthy', 'CRC')) %>%
  wilcox_test(Viruses ~ type, alternative = 'greater')

#test for difference in viral load: CRC vs paired
vir.pair <- kingdom_cts_m %>% 
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  wilcox_test(Viruses ~ type, paired = T, alternative = 'greater')

#test for difference in Archaea load: CRC vs healthy
arc.ind <- kingdom_cts_m %>% 
  filter(type %in% c('Healthy', 'CRC')) %>%
  wilcox_test(Archaea ~ type, alternative = 'greater')

#test for difference in Archaea load: CRC vs paired
arc.pair <- kingdom_cts_m %>% 
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  wilcox_test(Archaea ~ type, paired = T, alternative = 'greater')

pvals_all <- w.ind$p
pvals_all <- append(pvals_all, w.paired$p)
pvals_all <- append(pvals_all, vir.ind$p)
pvals_all <- append(pvals_all, vir.pair$p)
pvals_all <- append(pvals_all, arc.ind$p)
pvals_all <- append(pvals_all, arc.pair$p)
pvals_all <- p.adjust(pvals_all)
names(pvals_all) <- c('bact.ind.CRC.Healthy', 'bact.pair.CRC.Paired', 'vir.ind.CRC.Healthy',
                      'vir.pair.CRC.Paired', 'arc.ind.CRC.Healthy', 'arc.pair.CRC.Paired')
pvals_all


