library(tidyverse)
library(ggplot2)
library(ggrepel)
library(reshape)
library(phyloseq)
library(rstatix)
library(ggpubr)

#=======================
# Read in the metadata and count data
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

#this is the lowest amount of counts (approximator for amount of reads) across all files
min_cts <- min(metadata$tot_0)

dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
plot_dir <- "/media/henrike/E586-BCAE/Data_for_HeaDS/figures_HZ/"

###########################

#=======================
# Make OTU count tables: 
#=======================

# read in the filtered tables and assemble an OTU table were taxa are rows and samples are cols
# the relevant col from each sample df is new_est_reads. Turn that col into the col for the sample

#=======================
# testing
#=======================

#two different samples
d1 <- as_tibble(read.csv(paste0(dir_dat, 'La4.non_0_scaled.out'), sep = '\t'))
d1 <- dplyr::rename(d1, s41 = new_est_reads)

d2 <- as_tibble(read.csv(paste0(dir_dat, 'La10.non_0_scaled.out'), sep = '\t'))

# I need to merge on name and keep only the new_est_reads. col from each
dm <- merge(d1[,c('name', 's41')], d2[,c('name', 'new_est_reads')], by = 'name', all = T)

#re-name the last column: https://stackoverflow.com/questions/35999121/renaming-the-last-column-of-a-r-dataframe
names(dm)[ncol(dm)]<-"s9" 

#I would like tables for both scaled and original counts, so I will do both

#=======================
# in bulk
#=======================

# 4 is the first sample. load that to start the merging frame
first_sample <- as_tibble(read.csv(paste0(dir_dat, 'La4.non_0_scaled.out'), sep = '\t'))

dm_scaled <- dplyr::rename(first_sample, La4 = scaled_cts)
dm_scaled <- dm_scaled[,c('taxonomy_id', 'La4')]

dm_orig <- dplyr::rename(first_sample, La4 = new_est_reads)
dm_orig <- dm_orig[,c('taxonomy_id', 'La4')]

for (i in seq(5,124,1)) { 
  
  if(!(i %in% c(45,71,72))){
    d2 <- as_tibble(read.csv(paste0(dir_dat, 'La',i,'.non_0_scaled.out'), sep = '\t'))
    
    dm_scaled <- merge(dm_scaled, d2[,c('taxonomy_id', 'scaled_cts')], by = 'taxonomy_id', all = T)
    names(dm_scaled)[ncol(dm_scaled)]<-paste0('La',i)
    
    dm_orig <- merge(dm_orig, d2[,c('taxonomy_id', 'new_est_reads')], by = 'taxonomy_id', all = T)
    names(dm_orig)[ncol(dm_orig)]<-paste0('La',i)
    
  }
}


rownames(dm_scaled) <- dm_scaled$taxonomy_id
rownames(dm_orig) <- dm_orig$taxonomy_id

dm_scaled <- dm_scaled %>% select(-taxonomy_id)
dm_orig <- dm_orig %>% select(-taxonomy_id)

write.table(dm_scaled, paste0(dir_dat, 'OTU_count_table_scaled.out'), quote = F, row.names = T, sep = '\t')
write.table(dm_orig, paste0(dir_dat, 'OTU_count_table.out'), quote = F, row.names = T, sep = '\t')

#=======================
# detection threshold cutoff
#=======================

# we will used the scaled counts (i.e. counts between all samples are on the same scale) to determine the 
# detection threshold

#read in the scaled table again
OTU_count_table_scaled <- read.csv(paste0(dir_dat, 'OTU_count_table_scaled.out'), sep = '\t')
#re-add taxonomyID column
OTU_count_table_scaled$taxonomy_id <- rownames(OTU_count_table_scaled)

#hack to rm human so I can see the bact counts
non_hum <- OTU_count_table_scaled %>%
  filter(taxonomy_id != '9606')

# try to make one long vector with all count data so we can plot a histogram 
# and find a cutoff for when a call is believable
long <- melt(non_hum, id.vars = 'taxonomy_id')

p1 <- ggplot(long, aes(x=log10(value))) +
  geom_histogram(binwidth = 0.03) +
  theme_bw(base_size = 22) +
  scale_x_continuous(breaks = seq(-10,8,0.1)) +
  theme(axis.text.x = element_text(angle = 90)) +
  xlab('log10(scaled counts)')

png(filename = paste0(plot_dir, 'hist_scaled_cts_all_samples.png'), width = 2500, height = 1600)
p1
dev.off()

#the dip is at -0.9 which corresponds to about a scaled_cts value of 0.125: log10(0.125) = -0.903

#=======================
# apply filtering according to detection threshold
#=======================

OTU_count_table_scaled_filtered <- OTU_count_table_scaled %>%
  select(-taxonomy_id)
rownames(OTU_count_table_scaled_filtered) <- OTU_count_table_scaled$taxonomy_id

#set low counts to NA
OTU_count_table_scaled_filtered[OTU_count_table_scaled_filtered < 0.125] <- NA

#put back the taxIDs
#OTU_count_table_scaled_filtered$taxonomy_id <- OTU_count_table_scaled$taxonomy_id
#OTU_count_table_scaled_filtered <- OTU_count_table_scaled_filtered %>%
#  relocate(taxonomy_id)

#=======================
# side track
#=======================

#which samples have scaled counts < 0.125 and how many of them? Are they the low depth samples?
small_cts <- long %>%
  dplyr::filter(value < 0.125) %>%
  #distinct(variable)
  dplyr::count(variable)

small_cts <- merge(small_cts, metadata, by.x = 'variable', by.y = 'sample')

#No, these small counts occur across samples.

#=======================
# filter out the corresponding counts in the table with the original counts
#=======================

#load in the OTU table with original counts:
OTU_count_table <- read.csv(paste0(dir_dat, 'OTU_count_table.out'), sep = '\t')

# we must now apply the detection threshold we have found above to the original counts
# We do this by setting to NA all counts in OTU_count_table that are NA in OTU_count_table_scaled_filtered
# first, check that the rows and cols are in the same position between the two data frames
all(rownames(OTU_count_table) == rownames(OTU_count_table_scaled_filtered))
all(colnames(OTU_count_table) == colnames(OTU_count_table_scaled_filtered))

#get some examples for taxa filtered out:
OTU_count_table_scaled %>%
  select(La34,taxonomy_id) %>%
  filter(La34 < 0.125) %>%
  head()

# now apply the filtering:
OTU_count_table_filtered <- OTU_count_table
OTU_count_table_filtered[is.na(OTU_count_table_scaled_filtered)] <- NA

#get the count for taxID 41 for La34 in all tables:
OTU_count_table_scaled[c('41'),c('La34')]
OTU_count_table_scaled_filtered[c('41'),c('La34')]
OTU_count_table[c('41'),c('La34')]
OTU_count_table_filtered[c('41'),c('La34')]

#check if there are taxIds that now only have NA counts and remove these rows
#identify rows with only NA counts
which(rowSums(is.na(OTU_count_table_filtered))==ncol(OTU_count_table_filtered))
#drop rows with only NA counts
OTU_count_table_filtered <- OTU_count_table_filtered[!!rowSums(!is.na(OTU_count_table_filtered)),]

# and same for the scaled filtered table
which(rowSums(is.na(OTU_count_table_scaled_filtered))==ncol(OTU_count_table_scaled_filtered))
#drop rows with only NA counts
OTU_count_table_scaled_filtered <- OTU_count_table_scaled_filtered[!!rowSums(!is.na(OTU_count_table_scaled_filtered)),]

write.table(OTU_count_table_scaled_filtered, 
            paste0(dir_dat, 'OTU_count_table_scaled_filtered.out'), quote = F, row.names = T, sep = '\t')

write.table(OTU_count_table_filtered, 
            paste0(dir_dat, 'OTU_count_table_filtered.out'), quote = F, row.names = T, sep = '\t')


