library(tidyverse)
library(ggplot2)
library(ggrepel)
library(reshape)
library(phyloseq)
library(rstatix)
library(ggpubr)
library(pheatmap)

# look at composition with phyloseq to find what sets the CRC samples apart from Healthy 
# and Paired samples, in terms of microbial community

#some ideas:
# do CRC samples cluster in a PCA/ MDS?
# heatmaps
# distances: can we calc meaningful distances between samples and do the samples then cluster by type?

# ggplot: set theme to bw for the entire script
theme_set(theme_bw(base_size = 22))
#colors for the sample group
color_dict <- c('CRC' = '#D55E00', 'Healthy' = "#999999", 'Paired' = "#56B4E9")
depth_color_dict <- c('shallow' = '#E69F00', 'deep' = '#009E73')

#=======================
# Read in the metadata
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

#add seq depth factor as col
metadata <- metadata %>% 
  mutate(seq_depth = ifelse(tot_0 < 50000000, 'shallow', 'deep'))

dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
plot_dir <- "/media/henrike/E586-BCAE/Data_for_HeaDS/figures_HZ/"


#=======================
# Phyloseq
#=======================

# We use the filtered table of the orig counts since we use distance measures able to
# deal with difference in sample size
OTU_fo <- read.csv(paste0(dir_dat, 'OTU_count_table_filtered.out'), sep = '\t') 

#omit human since we're look at microbial composition
OTU_fo$taxID <- rownames(OTU_fo)
OTU_fo <- OTU_fo %>%
  filter(taxID != '9606') %>%
  select(-taxID)

# now load in the tax table and make phyloseq object
# according to documentation it should be a matrix with the relevant row and column names:
# https://joey711.github.io/phyloseq/import-data.html
a <- read.csv(paste0(dir_dat, 'tax_table.csv'), sep = '\t')
b <- a %>%
  select(-taxID)
c <- as.matrix(b)
rownames(c) <- paste0('taxID.',a$taxID)
colnames(c)
TAX = tax_table(c)

#perhaps having numbers as taxa distrubs phyloseq. Lets put letters in from of them:
rownames(OTU_fo) <- paste0('taxID.', rownames(OTU_fo))

#perhaps having NAs disturbs phyloseq. Lets replace them with 0's
OTU_fo[is.na(OTU_fo)] <- 0

#make phyloseq OTU table
OTU_o = otu_table(OTU_fo, taxa_are_rows = T)

#metadata
phylo_metadata <- sample_data(column_to_rownames(metadata, var = "sample"))

#phyloseq obj
physeq_orig_cts <- phyloseq(OTU_o, TAX, phylo_metadata)

#=======================
# Distances
#=======================

# Here is the tutorial for distances in phyloseq: https://joey711.github.io/phyloseq/distance.html
# We have a look at the available distance measures:
?distanceMethodList
# There are a lot! Lets look at those from the vegan package. It's a popular package
# for ecology/ microbiome.
?vegdist

# We'll use Morisita, Horn–Morisita, Binomial, Cao and Chao
dist_methods <- c('morisita', 'horn', 'binomial', 'cao', 'chao')

#=======================
# Example
#=======================
#calc one distance
i <- dist_methods[1]

iDist <- distance(physeq_orig_cts, method=i)
iDist
?pheatmap
dist_mat <- as.matrix(iDist)
# ok, iDist is a pairwise distance matrix and it has the sample names as row and col names

# lets make a heatmap showing the distances
pheatmap(dist_mat)

#add annotation about sample type and seq depth
annotation_row <- data.frame(sample_type = factor(metadata$type, ordered = T)) #CRC type
#according to https://stackoverflow.com/questions/51851868/error-in-check-lengthfill-gpar-element-fill-must-not-be-length-0
# colnames(mat) should be matched with rownames(df) of the annotation df
rownames(annotation_row) <- colnames(dist_mat)

annotation_col <- data.frame(depth = factor(metadata$seq_depth, ordered = T)) #seq depth
rownames(annotation_col) <- rownames(annotation_row)

pheatmap(dist_mat, 
         annotation_row = annotation_row,
         annotation_col = annotation_col,
         annotation_colors = list(
           sample_type = color_dict,
           depth = depth_color_dict), 
         labels_row = rep('',nrow(dist_mat)),
         labels_col = rep('',nrow(dist_mat)))


# Calculate ordination
iMDS  <- ordinate(physeq_orig_cts, "MDS", distance=iDist)
# What  is ordination? According to wikipedia it is:
# "More technically, MDS refers to a set of related ordination techniques used in information 
# visualization, in particular to display the information contained in a distance matrix. MDS 
# is used to translate "information about the pairwise 'distances' among a set of n objects or
# individuals" into a configuration of n points mapped into an abstract Cartesian space.[1]"

# So ordination can be used to transform an m x m distance matrix of m samples into a n-dim 
# space such that patterns are preserved as much as possible. (mostly n=2,3)

p <- plot_ordination(physeq_orig_cts, iMDS, color="type", shape="seq_depth")
p <- p + scale_color_manual(values=color_dict)
p <- p + geom_point(size = 4) 
p <- p + ggtitle(paste("MDS using distance method ", i, sep=""))
p

#=======================
# Now bulk
#=======================

# Alright, now we do this on all chosen distance methods and save the plots
distmat_list <- vector("list", length(dist_methods))
names(distmat_list) = dist_methods

# calc distance matrices
for( i in dist_methods ){
  print(i)
  iDist <- distance(physeq_orig_cts, method=i)
  distmat_list[[i]] <- iDist 
}

#heatmaps
for( i in dist_methods ){

  print(i)
  pheatmap(as.matrix(distmat_list[[i]]), 
           annotation_row = annotation_row,
           annotation_col = annotation_col,
           annotation_colors = list(
             sample_type = color_dict,
             depth = depth_color_dict),
           filename = paste0(plot_dir, 'heatmaps/', 'heat_', i, '.png'),
           width = 18, height = 9)
  
}

#MDS plots
for( i in dist_methods ){
  
  print(i)
  iDist <- distmat_list[[i]]
  # Calculate ordination
  iMDS  <- ordinate(physeq_orig_cts, "MDS", distance=iDist)
  ## Make plot
  # Don't carry over previous plot (if error, p will be blank)
  p <- NULL
  # Create plot, store as temp variable, p
  p <- plot_ordination(physeq_orig_cts, iMDS, color="type", shape="seq_depth")
  #p <- p + scale_colour_manual(values=c('CRC' = 'green', 'Healthy' = 'red', 'Paired' = 'blue'))
  #value from the cbPalette but manually assigned because I want CRC to be red-isch
  p <- p + scale_color_manual(values=color_dict)
  p <- p + geom_point(size = 4) 
  # Add title to each plot
  p <- p + ggtitle(paste("MDS using distance method ", i, sep=""))
  # add patient ID as labels
  p <- p + geom_text(mapping = aes(label = patient_ID), size = 5, vjust = 1.5)
  # Save the graphic to file.
  png(filename = paste0(plot_dir, 'MDS_plots/', 'MDS_dist_', i, '.png'), 
      width = 800, height = 600)
  print(p)
  dev.off()
}



