library(tidyverse)
library(ggplot2)
library(ggrepel)
library(taxize)
library(reshape)
library(phyloseq)
library(rstatix)
library(ggpubr)

# Abundance of B frag and F nuc

#=======================
# Read in the metadata and count data
#=======================

metadata <- read.delim("../metadata/metadata_update.tsv")

dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"
cts <- read.csv(paste0(dir_dat, 'OTU_count_table_scaled_filtered.out'), sep = '\t') 
tax <- read.csv(paste0(dir_dat, 'tax_table.csv'), sep = '\t')

cts[is.na(cts)] <-0 

# merge the taxonomy info to the counts so we can select the rows for the two bacteria
cts$taxID <- rownames(cts)
m <- merge(cts, tax, by = 'taxID', all.x = T)

#=======================
# Select F. nuc and B. frag data
#=======================

#select F. nuc
fnuc_cts <- m %>%
  filter(species == 'Fusobacterium nucleatum') %>%
  select(-taxID, -superkingdom, -phylum, -class, -order, -family, -genus, -species) %>%
  gather(variable, value) %>%
  dplyr::rename(sample = variable, fnuc_cts = value) %>%
  left_join(metadata, by = c("sample" = "sample"))

#check normality
ggqqplot(fnuc_cts, x = "fnuc_cts", facet.by = "type", main = 'Scaled counts F. nucleatum')

# there is a strong departure from normal distribution in CRC samples

#outliers
fnuc_cts %>%
  group_by(type) %>%
  identify_outliers(fnuc_cts) %>%
  relocate(is.outlier, is.extreme)

# We find extreme outliers in all 3 groups.

#select B. frag
bfrag_cts <- m %>%
  filter(species == 'Bacteroides fragilis') %>%
  select(-taxID, -superkingdom, -phylum, -class, -order, -family, -genus, -species) %>%
  gather(variable, value) %>%
  dplyr::rename(sample = variable, bfrag_cts = value) %>%
  left_join(metadata, by = c("sample" = "sample"))

#check normality
ggqqplot(bfrag_cts, x = "bfrag_cts", facet.by = "type", main = 'Scaled counts B. fragilis')

# there is a strong departure from normal distribution in CRC samples

#outliers
bfrag_cts %>%
  group_by(type) %>%
  identify_outliers(bfrag_cts) %>%
  relocate(is.outlier, is.extreme)


#=======================
# Wilcoxon tests
#=======================

# F.nucleatum

#independent tests between CRC Vs Healthy
crc_VS_healthy_fnuc <- fnuc_cts %>% 
  filter(type %in% c('CRC', 'Healthy')) %>%
  wilcox_test(fnuc_cts ~ type, alternative = 'greater')

#paired test CRC Vs Paired
crc_VS_paired_fnuc <- fnuc_cts %>%
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  wilcox_test(fnuc_cts ~ type, alternative = 'greater', paired = T) 

#B .fragilis

#independent tests between CRC Vs Healthy
crc_VS_healthy_bfrag <- bfrag_cts %>% 
  filter(type %in% c('CRC', 'Healthy')) %>%
  wilcox_test(bfrag_cts ~ type, alternative = 'greater')

#paired test CRC Vs Paired
crc_VS_paired_bfrag <- bfrag_cts %>%
  filter(type %in% c('CRC', 'Paired')) %>%
  filter(patient_ID != 54) %>%
  wilcox_test(bfrag_cts ~ type, alternative = 'greater', paired = T) 

# correct p-values
pvals <- c(crc_VS_healthy_fnuc$p, crc_VS_paired_fnuc$p,
           crc_VS_healthy_bfrag$p, crc_VS_paired_bfrag$p)
pvals <- p.adjust(pvals)
names(pvals) <- c('ind.fnuc.CRC.Healthy', 'pair.fnuc.CRC.Paired',
                  'ind.bfrag.CRC.Healthy', 'pair.bfrag.CRC.Paired')
pvals

#excursion into the Fusobacterium genus
###

bla <- m %>%
  filter(genus == 'Fusobacterium') %>%
  relocate(genus, species) %>%
  summarise(across(where(is.numeric), sum))


