library(tidyverse)
library(ggplot2)

### update the metadata 
# with info about the total read count and how many counts there are if we 
# consider all species assignments or only assignments with at least 10 reads
# and the scaling factor between each sample and the sample with the lowest
# amount of reads
# we do not have the actual number of reads but we will use the sum of bracken counts as an estimation

#=======================
# Read in the metadata
#=======================

metadata <- read.delim("../metadata/metadata_Rformatted.tsv")

# Extract Patient_ID 
metadata <- 
  metadata %>% 
  separate(pt_ID, sep = "-", into = c("Patient_ID")) #Get patient ID column 

#replace Lasse with La in the sample names
metadata$RNA_seq_ID <- gsub('Lasse','La',metadata$RNA_seq_ID)

#omit these two patients who were not part of the study
metadata <- metadata %>% 
  filter(Type != 'control' & !(Patient_ID %in% c('030', '055')))

# across the whole data, what is the difference in the sum of bracken counts if we only use species with >= 10 reads
# compared to use all species with at least 1 read. Do we severely underestimate the total number of reads by not counting
# those?

diffs <- c()
diffs_perc <- c()
tots_0 <- c()
tots_10 <- c()

#this is where the bracken counts files are
dir_dat <- "/media/henrike/E586-BCAE/Data_for_HeaDS/generated_data/bracken/"

# go through all samples
for (i in seq(4,124,1)) {
  
  if(!(i %in% c(45,71,72))){
    bracken_path <- paste0(dir_dat,"La", i, '.bracken.out')
    bracken_raw <- as_tibble(read.csv(bracken_path, sep = '\t'))
    
    bracken_gt0 <- bracken_raw %>%
      filter(new_est_reads > 0)
    
    bracken_gt10 <- bracken_raw %>%
      filter(new_est_reads >= 10)
    
    diff <- sum(bracken_gt0$new_est_reads)- sum(bracken_gt10$new_est_reads)
    diff_p <- diff * 100 / sum(bracken_gt0$new_est_reads)
    
    diffs <- append(diffs, diff)
    diffs_perc <- append(diffs_perc, diff_p)
    tots_0 <- append(tots_0, sum(bracken_gt0$new_est_reads))
    tots_10 <- append(tots_10, sum(bracken_gt10$new_est_reads))
    
  }
  
}

meta_all <- data.frame(sample = metadata$RNA_seq_ID, patient_ID = metadata$Patient_ID, 
                       type = metadata$Type, 
                       tot_0 = tots_0, tot_10 = tots_10, diff = diffs, diff_p = diffs_perc)

#this is the lowest amount of counts (approximator for amount of reads) across all files
min_cts <- min(meta_all$tot_0)

#Now, we can calc the scaling factor between the sample/min_cts
meta_all <- meta_all %>%
  mutate(scaling = min_cts/tot_0)

write.table(meta_all, '../metadata/metadata_update.tsv', quote = F, row.names = F, sep = '\t')

#=======================
# some visualization
#=======================

ggplot(meta_all, aes(y=tots_0,x='')) + geom_violin() + geom_jitter(width = 0.15)

ggplot(meta_all, aes(y=tots_10,x='')) + geom_violin() + geom_jitter(width = 0.15)

hist(diffs)
hist(diffs_perc)

