library(tidyverse)
library(taxize)

# How to get the tax string, i.e. full taxonomic classification from a tax ID using taxize package

dir_dat <- "~/Documents/Heads_center_management/datalab_projects/CRC_RNAseq_bacteria/Data_for_HeaDS/generated_data/bracken/"


#==========================================
# Example: 
#==========================================

#read in a count table
bracken_path <- paste0(dir_dat,"La", '10', '.bracken.out')
bracken_raw <- read.csv(bracken_path, sep = '\t')

bracken_hum <- bracken_raw %>%
  filter(kraken_assigned_reads > 0)

#try taxize on the first organism, homo sapiens
hum_class <- classification(bracken_hum[1,'taxonomy_id'], db = 'ncbi')

#and on a bacterium
bact_class <- classification(bracken_hum[2,'taxonomy_id'], db = 'ncbi')

#make a df from classification results so I can get the superkingdom out
a <- as.data.frame(unlist(bact_class, recursive = F))
colnames(a) = c('name', 'rank','taxid')

#which row has the superkingdom info?
a$rank == 'superkingdom'
#get that row:
a[a$rank == 'superkingdom',]
#get the name assigned to rank superkingdom (column 1)
a[a$rank == 'superkingdom',c('name')]
a

#==========================================
# Proof of concept: Create superkingdom vector
#==========================================

#read in the unique list of tax IDs
tax_ids <- read.csv(paste0(dir_dat, 'all_taxids'), header = T)
s_king_vector <- c()

for (i in seq(10)) { #test the first 10
  #look up the taxID from row i
  lookup <- classification(tax_ids[i,], db = 'ncbi')
  my_df <- as.data.frame(unlist(lookup, recursive = F))
  colnames(my_df) = c('name', 'rank','taxid')
  kingdom <- my_df[my_df$rank == 'superkingdom',c('name')]
  s_king_vector <- append(s_king_vector, kingdom)
  
  Sys.sleep(0.5)
  
}

#create dataframe by merging the superkingdoms we got to the tax IDs 
new_df <- data.frame(taxid = tax_ids[1:10,], kingdom = s_king_vector)

#==========================================
# Bulk:
#==========================================

#read in the unique list of tax IDs
tax_ids <- read.csv(paste0(dir_dat, 'all_taxids'), header = T)
colnames(tax_ids) <- c('taxID')

# for the ranks present in the tax table we need to agree which ranks we want to consider. 
# The seven official ones are: species, genus, family, order, class (clade), phylum, kingdom, domain (wikipedia)
# What I get back from ncbi via taxize is mostly: 
# superkingdom, phylum, class, order, family, genus, species
# so we will use these levels/ranks

ranks <- c('superkingdom', 'phylum', 'class', 'order', 'family', 'genus', 'species')

#bulk: create a vector that has the taxID, then the names for each rank or NA and 
#add it as a row to a growing tax table

#init the tax_table with the first tax ID and rank info
first_taxid <- tax_ids[1,]
first_taxid_class <- classification(first_taxid, db = 'ncbi')
first_taxid_df <- as.data.frame(unlist(first_taxid_class, recursive = F))
colnames(first_taxid_df) = c('name', 'rank','taxid')

# temporary vector to store information
tmp_vec <- c(first_taxid)
for (rank in ranks) {
  rank_name <- ifelse(rank %in% first_taxid_df$rank, first_taxid_df[first_taxid_df$rank == rank,c('name')], NA)
  tmp_vec <- append(tmp_vec, rank_name)
}

#init the tax table and add the first row
tax_table <- data.frame(taxID = numeric(),
                        superkingdom = character(),
                        phylum = character(),
                        class = character(),
                        order = character(),
                        family = character(),
                        genus = character(),
                        species = character(),
                        stringsAsFactors = F)
tax_table[1,] <- tmp_vec

outfile <- paste0(dir_dat, 'tax_table.csv')
write.table(tax_table, outfile, quote = F, row.names = F, sep = '\t')

# now query all other tax IDs
# since these are API queries that can and do fail we employ the following strategy:
# 1. load in the current tax_table.csv
# 2. query the current tax ID
# 3. if successful, add to the tax table and re-save new tax_table which is one row longer now
# 4. repeat

# when querying fails, restart the loop from the last tax ID that failed which will 
# be in the same position as the current length of tax_table.csv

for (i in seq(1,nrow(tax_ids))) {  #replace 1 to start from the last taxid that failed
  #for (i in seq(2,10)) { #test the first 10, excluding the first
  
  outfile <- paste0(dir_dat, 'tax_table.csv')
  
  #read in the already written tax_table
  tax_table <- read.csv(outfile, sep = '\t')
  
  #tell which ID we are looking at:
  print(tax_ids[i,])
  
  tmp_vec <- c(tax_ids[i,])
  #look up the taxID from row i
  lookup <- classification(tax_ids[i,], db = 'ncbi')
  my_df <- as.data.frame(unlist(lookup, recursive = F))
  colnames(my_df) = c('name', 'rank','taxid')
  for (rank in ranks) {
    rank_name <- ifelse(rank %in% my_df$rank, my_df[my_df$rank == rank,c('name')], NA)
    tmp_vec <- append(tmp_vec, rank_name)
  }
  
  #add to the tax table
  tax_table <- rbind(tax_table, tmp_vec)
  
  #save the current tax_table
  write.table(tax_table, outfile, quote = F, row.names = F, sep = '\t')
  
  Sys.sleep(1)
  
}





